from collections.abc import Callable

from nonebot.permission import Permission

from .params import Session, UniSession


async def _private(sess: Session | None = UniSession()) -> bool:
    if not sess:
        return False
    return sess.scene.is_private


PRIVATE: Permission = Permission(_private)
""" 匹配任意私聊类型事件"""


async def _group(sess: Session | None = UniSession()) -> bool:
    if not sess:
        return False
    return sess.scene.is_group


GROUP: Permission = Permission(_group)
"""匹配任意群聊类型事件"""


async def _guild(sess: Session | None = UniSession()) -> bool:
    if not sess:
        return False
    return sess.scene.is_guild or sess.scene.is_channel


GUILD: Permission = Permission(_guild)
"""匹配任意频道消息类型事件"""


def ROLE_IN(role_id: str, *role_ids: str) -> Permission:
    """检查成员是否在指定角色组中"""

    async def _role_in(sess: Session | None = UniSession()) -> bool:
        if not sess or not sess.member or not sess.member.roles:
            return False
        member_role_ids = {role.id for role in sess.member.roles}
        _role_ids = {role_id, *role_ids}
        return not member_role_ids.isdisjoint(_role_ids)

    return Permission(_role_in)


def ROLE_NOT_IN(role_id: str, *role_ids: str) -> Permission:
    """检查成员是否不在指定角色组中"""

    async def _role_not_in(sess: Session | None = UniSession()) -> bool:
        if not sess or not sess.member or not sess.member.roles:
            return True
        member_role_ids = {role.id for role in sess.member.roles}
        _role_ids = {role_id, *role_ids}
        return member_role_ids.isdisjoint(_role_ids)

    return Permission(_role_not_in)


def MEMBER() -> Permission:
    return ROLE_NOT_IN("CHANNEL_ADMINISTRATOR", "ADMINISTRATOR", "OWNER")


def ADMIN() -> Permission:
    return ROLE_IN("ADMINISTRATOR", "OWNER")


def OWNER() -> Permission:
    return ROLE_IN("OWNER")


def ROLE_LEVEL(checker: Callable[[int], bool]) -> Permission:
    """检查用户角色等级"""

    async def _level(sess: Session | None = UniSession()) -> bool:
        if not sess or not sess.member or not sess.member.roles:
            return False
        max_level = max((role.level for role in sess.member.roles), default=0)
        return checker(max_level)

    return Permission(_level)


def USER_IN(user_id: str, *user_ids: str) -> Permission:
    """检查用户是否在指定用户中"""

    async def _user_in(sess: Session | None = UniSession()) -> bool:
        if not sess:
            return False
        return sess.user.id in (user_id, *user_ids)

    return Permission(_user_in)


def USER_NOT_IN(user_id: str, *user_ids: str) -> Permission:
    """检查用户是否不在指定用户中"""

    async def _user_not_in(sess: Session | None = UniSession()) -> bool:
        if not sess:
            return True
        return sess.user.id not in (user_id, *user_ids)

    return Permission(_user_not_in)


def SCENE_IN(scene_id: str, *scene_ids: str) -> Permission:
    """检查场景是否在指定场景中"""

    async def _scene_in(sess: Session | None = UniSession()) -> bool:
        if not sess:
            return False
        return sess.scene.id in (scene_id, *scene_ids)

    return Permission(_scene_in)


def SCENE_NOT_IN(scene_id: str, *scene_ids: str) -> Permission:
    """检查场景是否不在指定场景中"""

    async def _scene_not_in(sess: Session | None = UniSession()) -> bool:
        if not sess:
            return True
        return sess.scene.id not in (scene_id, *scene_ids)

    return Permission(_scene_not_in)


__all__ = [
    "PRIVATE",
    "GUILD",
    "GROUP",
    "ROLE_IN",
    "ROLE_NOT_IN",
    "MEMBER",
    "ADMIN",
    "OWNER",
    "USER_IN",
    "USER_NOT_IN",
    "SCENE_IN",
    "SCENE_NOT_IN",
    "ROLE_LEVEL",
]



"""constraint_name

迁移 ID: 7d23eb54c6be
父迁移: 14175fde8186
创建时间: 2025-01-01 20:54:51.070023

"""

from __future__ import annotations

from collections.abc import Sequence

from alembic import op

revision: str = "7d23eb54c6be"
down_revision: str | Sequence[str] | None = "14175fde8186"
branch_labels: str | Sequence[str] | None = None
depends_on: str | Sequence[str] | None = None


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("nonebot_plugin_uninfo_botmodel", schema=None) as batch_op:
        batch_op.drop_constraint(
            "unique_bot",
            type_="unique",
        )
        batch_op.create_unique_constraint(
            "nonebot_plugin_uninfo_unique_bot",
            ["self_id", "adapter"],
        )

    with op.batch_alter_table("nonebot_plugin_uninfo_scenemodel", schema=None) as batch_op:
        batch_op.drop_constraint(
            "unique_scene",
            type_="unique",
        )
        batch_op.create_unique_constraint(
            "nonebot_plugin_uninfo_unique_scene",
            ["bot_persist_id", "scene_id", "scene_type"],
        )

    with op.batch_alter_table("nonebot_plugin_uninfo_usermodel", schema=None) as batch_op:
        batch_op.drop_constraint(
            "unique_user",
            type_="unique",
        )
        batch_op.create_unique_constraint(
            "nonebot_plugin_uninfo_unique_user",
            ["bot_persist_id", "user_id"],
        )

    with op.batch_alter_table("nonebot_plugin_uninfo_sessionmodel", schema=None) as batch_op:
        batch_op.drop_constraint(
            "unique_session",
            type_="unique",
        )
        batch_op.create_unique_constraint(
            "nonebot_plugin_uninfo_unique_session",
            ["bot_persist_id", "scene_persist_id", "user_persist_id"],
        )
    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("nonebot_plugin_uninfo_sessionmodel", schema=None) as batch_op:
        batch_op.drop_constraint(
            "nonebot_plugin_uninfo_unique_session",
            type_="unique",
        )
        batch_op.create_unique_constraint(
            "unique_session",
            ["bot_persist_id", "scene_persist_id", "user_persist_id"],
        )

    with op.batch_alter_table("nonebot_plugin_uninfo_usermodel", schema=None) as batch_op:
        batch_op.drop_constraint(
            "nonebot_plugin_uninfo_unique_user",
            type_="unique",
        )
        batch_op.create_unique_constraint(
            "unique_user",
            ["bot_persist_id", "user_id"],
        )

    with op.batch_alter_table("nonebot_plugin_uninfo_scenemodel", schema=None) as batch_op:
        batch_op.drop_constraint(
            "nonebot_plugin_uninfo_unique_scene",
            type_="unique",
        )
        batch_op.create_unique_constraint(
            "unique_scene",
            ["bot_persist_id", "scene_id", "scene_type"],
        )

    with op.batch_alter_table("nonebot_plugin_uninfo_botmodel", schema=None) as batch_op:
        batch_op.drop_constraint(
            "nonebot_plugin_uninfo_unique_bot",
            type_="unique",
        )
        batch_op.create_unique_constraint(
            "unique_bot",
            ["self_id", "adapter"],
        )
    # ### end Alembic commands ###



from pydantic import BaseModel, Field


class Config(BaseModel):
    """Plugin Config Here"""

    uninfo_cache: bool = Field(default=True, description="是否启用缓存")
    """是否启用缓存"""

    uninfo_cache_expire: int = Field(default=300, description="缓存过期时间")
    """缓存过期时间"""



from dataclasses import asdict, dataclass, field
from datetime import datetime, timedelta
from enum import IntEnum
import json
from typing import Any, Optional, TypedDict, TypeVar
from typing_extensions import Required

from nonebot.compat import DEFAULT_CONFIG, PYDANTIC_V2

from .constraint import SupportAdapter, SupportScope
from .util import DatetimeJsonEncoder


class BasicInfo(TypedDict):
    self_id: Required[str]
    adapter: Required[SupportAdapter]
    scope: Required[SupportScope]


class SceneType(IntEnum):
    PRIVATE = 0
    """私聊场景"""
    GROUP = 1
    """群聊场景"""
    GUILD = 2
    """频道场景"""
    CHANNEL_TEXT = 3
    """子频道文本场景"""
    CHANNEL_CATEGORY = 4
    """频道分类场景"""
    CHANNEL_VOICE = 5
    """子频道语音场景"""


C = TypeVar("C")


def _apply_schema(cls: type[C]) -> type[C]:
    if PYDANTIC_V2:
        from pydantic import VERSION
        from pydantic._internal._config import ConfigWrapper
        from pydantic._internal._dataclasses import complete_dataclass

        origin_init = cls.__init__
        origin_post_init = getattr(cls, "__post_init__", None)
        if int(VERSION.split(".")[1]) >= 10:
            complete_dataclass(cls, ConfigWrapper(DEFAULT_CONFIG, check=False))  # type: ignore
        else:
            complete_dataclass(cls, ConfigWrapper(DEFAULT_CONFIG, check=False), types_namespace=None)  # type: ignore
        cls.__init__ = origin_init  # type: ignore
        if origin_post_init:
            cls.__post_init__ = origin_post_init  # type: ignore
    else:
        from pydantic.dataclasses import _add_pydantic_validation_attributes

        origin_init = cls.__init__
        origin_post_init = getattr(cls, "__post_init__", None)
        _add_pydantic_validation_attributes(cls, DEFAULT_CONFIG, False, cls.__doc__ or "")
        cls.__init__ = origin_init  # type: ignore
        if origin_post_init:
            cls.__post_init__ = origin_post_init  # type: ignore
        cls.__pydantic_model__.update_forward_refs()
    return cls


class ModelMixin:

    @classmethod
    def load(cls, data: dict):
        return cls(**data)  # type: ignore  # noqa

    def dump(self) -> dict[str, Any]:
        return json.loads(self.dump_json())

    def dump_json(self, indent: int | None = None) -> str:
        return json.dumps(asdict(self), ensure_ascii=False, indent=indent, cls=DatetimeJsonEncoder)  # type: ignore  # noqa


class HashableMixin:
    id: str

    def __hash__(self) -> int:
        return hash(self.id)

    def __eq__(self, other: object) -> bool:
        return isinstance(other, type(self)) and self.id == other.id


@_apply_schema
@dataclass
class Scene(ModelMixin, HashableMixin):
    """对话场景，如群组、频道、私聊等"""

    id: str
    """场景id"""
    type: SceneType
    """场景类型"""
    name: str | None = None
    """场景名称"""
    avatar: str | None = None
    """场景头像"""
    parent: Optional["Scene"] = None
    """父级场景"""

    @property
    def is_private(self) -> bool:
        return self.type == SceneType.PRIVATE

    @property
    def is_group(self) -> bool:
        return self.type == SceneType.GROUP

    @property
    def is_guild(self) -> bool:
        return self.type == SceneType.GUILD

    @property
    def is_channel(self) -> bool:
        return self.type.value >= SceneType.CHANNEL_TEXT.value

    @classmethod
    def load(cls, data: dict):
        _data = data.copy()
        _data["type"] = SceneType(data["type"])
        if data.get("parent"):
            _data["parent"] = cls.load(data["parent"])
        return cls(**_data)


@_apply_schema
@dataclass
class User(ModelMixin, HashableMixin):
    """用户信息"""

    id: str
    """用户id"""
    name: str | None = None
    """用户名"""
    nick: str | None = None
    """用户昵称"""
    avatar: str | None = None
    """用户头像"""
    gender: str = "unknown"
    """用户性别"""


@_apply_schema
@dataclass
class Role(ModelMixin):
    """群员角色信息"""

    id: str
    """角色id"""
    level: int = 0
    """角色等级/权限"""
    name: str | None = None
    """角色名称"""


@_apply_schema
@dataclass
class MuteInfo(ModelMixin):
    """禁言信息"""

    muted: bool
    """是否被禁言"""
    duration: timedelta
    """禁言时长"""
    start_at: datetime | None = None
    """禁言开始时间"""

    @classmethod
    def load(cls, data: dict):
        _data = data.copy()
        _data["duration"] = timedelta(seconds=data["duration"])
        if data.get("start_at"):
            _data["start_at"] = datetime.fromtimestamp(data["start_at"])
        return cls(**_data)

    def __post_init__(self):
        if self.duration.total_seconds() < 1:
            self.muted = False
        if self.start_at and (datetime.now() - self.start_at) > self.duration:
            self.muted = False


@_apply_schema
@dataclass
class Member(ModelMixin):
    """群员信息"""

    user: User
    """群员用户信息"""
    nick: str | None = None
    """群员昵称"""
    mute: MuteInfo | None = None
    """群员禁言信息"""
    joined_at: datetime | None = None
    """加入时间"""
    roles: list[Role] = field(default_factory=list)
    """群员角色"""

    @property
    def id(self) -> str:
        return self.user.id

    @property
    def role(self) -> Role | None:
        """获取权限最高的角色"""
        if not self.roles:
            return None
        return max(self.roles, key=lambda r: r.level)

    @classmethod
    def load(cls, data: dict):
        _data = data.copy()
        _data["user"] = User.load(data["user"])
        if data.get("role"):
            _data["roles"] = [Role.load(data["role"])]
        if data.get("roles"):
            _data["roles"] = [Role.load(role) for role in data["roles"]]
        if data.get("mute"):
            _data["mute"] = MuteInfo.load(data["mute"])
        if data.get("joined_at"):
            _data["joined_at"] = datetime.fromtimestamp(data["joined_at"])
        return cls(**_data)


@_apply_schema
@dataclass
class Session(ModelMixin, HashableMixin):
    """对话信息"""

    self_id: str
    """机器人id"""
    adapter: str | SupportAdapter
    """适配器名称"""
    scope: str | SupportScope
    """适配器范围，相比 adapter 更指向实际平台"""
    scene: Scene
    """场景信息"""
    user: User
    """用户信息"""
    member: Member | None = None
    """群员信息"""
    operator: Member | None = None
    """操作者信息"""
    platform: str | set[str] | None = None
    """平台名称，仅当目标适配器存在多个平台时使用"""

    @property
    def id(self) -> str:
        """会话唯一标识符"""
        if self.scene.is_private:
            return self.scene_path
        return f"{self.scene_path}_{self.user.id}"

    @property
    def scene_path(self) -> str:
        """会话的场景路径，类似于 `event.get_session_id()`"""
        if self.scene.is_private:
            if self.scene.parent:
                return f"{self.scene.parent.id}_{self.user.id}"
            return self.user.id
        if self.scene.is_group:
            return self.scene.id
        if self.scene.parent:
            return f"{self.scene.parent.id}_{self.scene.id}"
        return self.scene.id

    @property
    def guild(self) -> Scene | None:
        """父级频道"""
        if self.scene.is_guild:
            return self.scene
        elif self.scene.is_channel:
            return self.scene.parent

    @property
    def channel(self) -> Scene | None:
        """子频道"""
        if self.scene.is_channel:
            return self.scene

    @property
    def group(self) -> Scene | None:
        """群组"""
        if self.scene.is_group:
            return self.scene

    @property
    def friend(self) -> Scene | None:
        """好友"""
        if self.scene.is_private:
            return self.scene

    @property
    def basic(self) -> BasicInfo:
        return {"self_id": self.self_id, "adapter": SupportAdapter(self.adapter), "scope": SupportScope(self.scope)}

    @classmethod
    def load(cls, data: dict):
        _data = data.copy()
        _data["adapter"] = SupportAdapter(data["adapter"])
        _data["scope"] = SupportScope(data["scope"])
        _data["scene"] = Scene.load(data["scene"])
        _data["user"] = User.load(data["user"])
        if data.get("member"):
            _data["member"] = Member.load(data["member"])
        if data.get("operator"):
            _data["operator"] = Member.load(data["operator"])
        return cls(**_data)



from nonebot.adapters.console import Bot
from nonebot.adapters.console.event import Event
from nonechat.model import DIRECT

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Scene, SceneType, User


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(data['avatar']):x}.png",
        )

    def extract_scene(self, data):
        if "channel_id" in data:
            return Scene(
                id=data["channel_id"],
                type=SceneType.GROUP,
                name=data["channel_name"],
                avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(data['channel_avatar']):x}.png",
            )
        return Scene(
            id=f"private:{data['user_id']}",
            type=SceneType.PRIVATE,
            name=data["name"],
            avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(data['avatar']):x}.png",
        )

    def extract_member(self, data, user: User | None):
        if user is None:
            user = self.extract_user(data)
        return Member(user, user.name)

    async def query_user(self, bot: Bot, user_id: str):
        if user_id == bot.self_id:
            return User(
                bot.self_id,
                bot.info.nickname,
                avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(bot.info.avatar):x}.png",
            )
        try:
            user = await bot.get_user(user_id)
            return User(
                user.id,
                user.nickname,
                avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(user.avatar):x}.png",
            )
        except Exception:
            return User(user_id)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            if user := await self.query_user(bot, scene_id):
                return Scene(id=f"private:{user.id}", type=SceneType.PRIVATE, name=user.name, avatar=user.avatar)
        if scene_type == SceneType.GROUP:
            try:
                channel = await bot.get_channel(scene_id)
                return Scene(
                    id=channel.id,
                    type=SceneType.GROUP,
                    name=channel.name,
                    avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(channel.avatar):x}.png",
                )
            except Exception:
                return Scene(id=scene_id, type=SceneType.GROUP)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        return Member(await self.query_user(bot, user_id))

    async def query_users(self, bot: Bot):
        for user in await bot.list_users():
            yield User(
                user.id,
                user.nickname,
                avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(user.avatar):x}.png",
            )

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is None or scene_type == SceneType.PRIVATE:
            for user in await bot.list_users():
                yield Scene(
                    id=f"private:{user.id}",
                    type=SceneType.PRIVATE,
                    name=user.nickname,
                    avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(user.avatar):x}.png",
                )
        if scene_type is None or scene_type == SceneType.GROUP:
            for channel in await bot.list_channels():
                yield Scene(
                    id=channel.id,
                    type=SceneType.GROUP,
                    name=channel.name,
                    avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(channel.avatar):x}.png",
                )

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        for user in await bot.list_users():
            yield Member(
                user=User(
                    user.id,
                    user.nickname,
                    avatar=f"https://emoji.aranja.com/static/emoji-data/img-apple-160/{ord(user.avatar):x}.png",
                ),
                nick=user.nickname,
            )

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.console,
            "scope": SupportScope.console,
        }


fetcher = InfoFetcher(SupportAdapter.console)


@fetcher.supply_wildcard
async def _(bot: Bot, event: Event):
    base = {"user_id": event.user.id, "name": event.user.nickname, "avatar": event.user.avatar}
    if event.channel.id == DIRECT.id or event.channel.id.startswith("private:"):
        return base
    return base | {
        "channel_id": event.channel.id,
        "channel_name": event.channel.name,
        "channel_avatar": event.channel.avatar,
    }



from nonebot.adapters.efchat import Bot
from nonebot.adapters.efchat.event import (
    ChannelMessageEvent,
    InviteEvent,
    WhisperMessageEvent,
)

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Scene, SceneType, User


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(id=data["user_id"], name=data["user_id"], avatar=data.get("head"))

    def extract_scene(self, data):
        if "channel_id" in data:
            return Scene(
                id=data["channel_id"],
                type=SceneType.GROUP,
                name=data["channel_id"],
            )
        return Scene(id=data["user_id"], type=SceneType.PRIVATE, name=data["user_id"], avatar=data.get("head", None))

    def extract_member(self, data, user: User | None):
        if user is None:
            user = self.extract_user(data)
        return Member(user, user.name)

    async def query_user(self, bot: Bot, user_id: str):
        return User(user_id, user_id)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        return Scene(id=scene_id, type=scene_type, name=scene_id, avatar=None)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        return Member(await self.query_user(bot, user_id), user_id)

    def query_users(self, bot: Bot):
        raise NotImplementedError

    def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        raise NotImplementedError

    def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        raise NotImplementedError

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.efchat,
            "scope": SupportScope.efchat,
        }


fetcher = InfoFetcher(SupportAdapter.efchat)


@fetcher.supply
async def _(bot: Bot, event: ChannelMessageEvent):
    return {"user_id": event.nick, "channel_id": event.channel, "head": event.head}


@fetcher.supply
async def _(bot: Bot, event: WhisperMessageEvent):
    return {
        "user_id": event.nick,
    }


@fetcher.supply
async def _(bot: Bot, event: InviteEvent):
    return {"user_id": event.nick, "channel_id": event.to}


@fetcher.supply_wildcard
async def _(bot: Bot, event):
    if hasattr(event, "nick"):
        return {"user_id": event.nick, "channel_id": bot.cfg.channel, "head": getattr(event, "head", None)}
    raise NotImplementedError



from typing import Any

from nonebot.adapters.yunhu import Bot
from nonebot.adapters.yunhu.event import (
    BotFollowedNoticeEvent,
    BotUnfollowedNoticeEvent,
    ButtonReportNoticeEvent,
    GroupJoinNoticeEvent,
    GroupLeaveNoticeEvent,
    GroupMessageEvent,
    InstructionMessageEvent,
    PrivateMessageEvent,
    TipNoticeEvent,
)
from nonebot.compat import model_dump
from nonebot.exception import ActionFailed

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Role, Scene, SceneType, User

ROLES = {
    "owner": ("OWNER", 100),
    "administrator": ("ADMINISTRATOR", 10),
    "member": ("MEMBER", 1),
    "unknown": ("UNKNOWN", 0),
}


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["userId"],
            name=data["nickname"],
            avatar=data["avatarUrl"],
        )

    def extract_scene(self, data):
        if "groupId" in data and data["groupId"]:
            return Scene(
                id=data["groupId"],
                type=SceneType.GROUP,
                name=data["name"],
                avatar=data["groupAvatarUrl"] if "groupAvatarUrl" in data else data["avatarUrl"],
            )
        return Scene(id=data["userId"], type=SceneType.PRIVATE, name=data["nickname"], avatar=data["avatarUrl"])

    def extract_member(self, data: dict[str, Any], user: User | None) -> Member | None:
        if user:
            return Member(user, roles=[Role(*ROLES[data["role"]], data["role"])] if "role" in data else [])
        return Member(
            self.extract_user(data), roles=[Role(*ROLES[data["role"]], data["role"])] if "role" in data else []
        )

    async def query_user(self, bot: Bot, user_id: str):
        data = await bot.get_user_info(user_id=user_id)
        if data.data:
            return self.extract_user(model_dump(data.data.user))

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            if user := await self.query_user(bot, scene_id):
                data = {
                    "userId": user.id,
                    "nickname": user.name,
                    "avatarUrl": user.avatar,
                }
                return self.extract_scene(data)
        elif scene_type == SceneType.GROUP:
            chat = await bot.get_group_info(scene_id)
            if chat.data:
                return self.extract_scene(model_dump(chat.data.group))

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type == SceneType.GROUP:
            member = await bot.get_user_info(user_id)
            if member.data:
                return self.extract_member(model_dump(member.data.user), None)

    def query_users(self, bot: Bot):
        raise NotImplementedError

    def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        raise NotImplementedError

    def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        raise NotImplementedError

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": bot.self_id,
            "adapter": SupportAdapter.yunhu,
            "scope": SupportScope.yunhu,
        }


fetcher = InfoFetcher(SupportAdapter.yunhu)


@fetcher.supply
async def _(bot: Bot, event: PrivateMessageEvent):
    return {
        "userId": event.event.sender.senderId,
        "nickname": event.event.sender.senderNickname,
        "avatarUrl": event.event.sender.senderAvatarUrl,
    }


@fetcher.supply
async def _(bot: Bot, event: GroupMessageEvent):
    try:
        group = await bot.get_group_info(event.event.chat.chatId)
        if g := group.data:
            group_info = {
                "name": g.group.name,
                "groupAvatarUrl": g.group.avatarUrl,
            }
        else:
            group_info = {}
    except ActionFailed:
        group_info = {}
    return {
        "userId": event.event.sender.senderId,
        "nickname": event.event.sender.senderNickname,
        "avatarUrl": event.event.sender.senderAvatarUrl,
        "role": event.event.sender.senderUserLevel,
        "groupId": event.event.chat.chatId,
        "name": group_info.get("name"),
        "groupAvatarUrl": group_info.get("groupAvatarUrl"),
    }


@fetcher.supply
async def _(bot: Bot, event: InstructionMessageEvent):
    try:
        if event.event.chat.chatType == "group":
            group = await bot.get_group_info(event.event.chat.chatId)
            if g := group.data:
                group_info = {
                    "name": g.group.name,
                    "groupAvatarUrl": g.group.avatarUrl,
                    "groupId": event.event.chat.chatId,
                }
            else:
                group_info = {}
        else:
            group_info = {}
    except ActionFailed:
        group_info = {}
    return {
        "userId": event.event.sender.senderId,
        "nickname": event.event.sender.senderNickname,
        "avatarUrl": event.event.sender.senderAvatarUrl,
        "groupId": group_info.get("groupId"),
        "name": group_info.get("name"),
        "groupAvatarUrl": group_info.get("groupAvatarUrl"),
    }


@fetcher.supply
async def _(bot: Bot, event: BotFollowedNoticeEvent | BotUnfollowedNoticeEvent):
    return {
        "userId": event.event.userId,
        "nickname": event.event.nickname,
        "avatarUrl": event.event.avatarUrl,
    }


@fetcher.supply
async def _(bot: Bot, event: GroupJoinNoticeEvent | GroupLeaveNoticeEvent):
    try:
        group = await bot.get_group_info(event.event.chatId)
        if g := group.data:
            group_info = {
                "name": g.group.name,
                "groupAvatarUrl": g.group.avatarUrl,
            }
        else:
            group_info = {}
    except ActionFailed:
        group_info = {}
    return {
        "userId": event.event.userId,
        "nickname": event.event.nickname,
        "avatarUrl": event.event.avatarUrl,
        "groupId": event.event.chatId,
        "name": group_info.get("name"),
        "groupAvatarUrl": group_info.get("groupAvatarUrl"),
    }


@fetcher.supply
async def _(bot: Bot, event: TipNoticeEvent):
    try:
        group = await bot.get_group_info(event.event.chatId)
        if g := group.data:
            group_info = {
                "name": g.group.name,
                "groupAvatarUrl": g.group.avatarUrl,
            }
        else:
            group_info = {}
    except ActionFailed:
        group_info = {}
    return {
        "userId": event.event.userId,
        "nickname": event.event.sender.senderNickname,
        "avatarUrl": event.event.sender.senderAvatarUrl,
        "groupId": event.event.chatId,
        "name": group_info.get("name"),
        "groupAvatarUrl": group_info.get("groupAvatarUrl"),
    }


@fetcher.supply
async def _(bot: Bot, event: ButtonReportNoticeEvent):
    try:
        if event.event.chatType == "group":
            group = await bot.get_group_info(event.event.chatId)
            if g := group.data:
                group_info = {
                    "name": g.group.name,
                    "groupAvatarUrl": g.group.avatarUrl,
                    "groupId": event.event.chatId,
                }
            else:
                group_info = {}
        else:
            group_info = {}
    except ActionFailed:
        group_info = {}
    try:
        user = await bot.get_user_info(event.event.userId)
        if u := user.data:
            user_info = {
                "nickname": u.user.nickname,
                "avatarUrl": u.user.avatarUrl,
            }
        else:
            user_info = {}
    except ActionFailed:
        user_info = {}
    return {
        "userId": event.event.userId,
        "nickname": user_info.get("nickname"),
        "avatarUrl": user_info.get("avatarUrl"),
        "groupId": group_info.get("groupId"),
        "name": group_info.get("name"),
        "groupAvatarUrl": group_info.get("groupAvatarUrl"),
    }



import sys

from nonebot.utils import logger_wrapper

log = logger_wrapper("Plugin-Uninfo")

if sys.version_info >= (3, 11):
    from enum import Enum, StrEnum
else:
    from enum import Enum

    class StrEnum(str, Enum):
        pass


class SupportAdapter(StrEnum):
    """支持的适配器"""

    bililive = "bilibili Live"
    console = "Console"
    discord = "Discord"
    dodo = "DoDo"
    efchat = "EFChat"
    feishu = "Feishu"
    # gewechat = "gewechat"
    kritor = "Kritor"
    kook = "Kaiheila"
    mail = "Mail"
    minecraft = "Minecraft"
    mirai = "Mirai"
    milky = "Milky"
    onebot11 = "OneBot V11"
    onebot12 = "OneBot V12"
    qq = "QQ"
    satori = "Satori"
    telegram = "Telegram"
    # tail_chat = "Tailchat"  # TODO: Add tailchat support
    wxmp = "WXMP"
    yunhu = "YunHu"

    nonebug = "fake"


class SupportScope(StrEnum):
    """支持的平台范围"""

    qq_client = "QQClient"
    """QQ 协议端"""
    qq_guild = "QQGuild"
    """QQ 用户频道，非官方接口"""
    qq_api = "QQAPI"
    """QQ 官方接口"""
    telegram = "Telegram"
    discord = "Discord"
    feishu = "Feishu"
    dodo = "DoDo"
    kook = "Kaiheila"
    mail = "Mail"
    minecraft = "Minecraft"
    github = "GitHub"
    console = "Console"
    ding = "Ding"
    wechat = "WeChat"
    """微信平台"""
    wechat_oap = "WeChatOfficialAccountPlatform"
    """微信公众号平台"""
    wecom = "WeCom"
    """企业微信平台"""
    tail_chat = "TailChat"
    """Tailchat平台"""
    efchat = "EFChat"
    """EFChat平台"""
    bililive = "BiliLive"
    """Bilibili直播平台"""
    yunhu = "YunHu"
    """云湖平台"""

    onebot12_other = "Onebot12"
    """ob12 的其他平台"""
    satori_other = "Satori"
    """satori 的其他平台"""

    unknown = "Unknown"
    """未知平台"""

    @staticmethod
    def ensure_ob12(platform: str):
        return {
            "qq": SupportScope.qq_client,
            "qqguild": SupportScope.qq_guild,
            "discord": SupportScope.discord,
            "wechat": SupportScope.wechat,
            "kaiheila": SupportScope.kook,
        }.get(platform, SupportScope.onebot12_other)

    @staticmethod
    def ensure_satori(platform: str):
        return {
            "red": SupportScope.qq_client,
            "chronocat": SupportScope.qq_client,
            "onebot": SupportScope.qq_client,
            "nekobox": SupportScope.qq_client,
            "lagrange": SupportScope.qq_client,
            "lagrange.python": SupportScope.qq_client,
            "milky": SupportScope.qq_client,
            "lagrange.milky": SupportScope.qq_client,
            "qq": SupportScope.qq_api,
            "qqguild": SupportScope.qq_api,
            "telegram": SupportScope.telegram,
            "discord": SupportScope.discord,
            "feishu": SupportScope.feishu,
            "gewechat": SupportScope.wechat,
            "wechat-official": SupportScope.wechat_oap,
            "wecom": SupportScope.wecom,
            "kook": SupportScope.kook,
            "dingtalk": SupportScope.ding,
            "mail": SupportScope.mail,
            "yunhu": SupportScope.yunhu,
        }.get(platform, SupportScope.satori_other)


class SupportAdapterModule(str, Enum):
    """支持的适配器的模块路径"""

    bililive = "nonebot.adapters.bilibili_live"
    console = "nonebot.adapters.console"
    discord = "nonebot.adapters.discord"
    dodo = "nonebot.adapters.dodo"
    efchat = "nonebot.adapters.efchat"
    feishu = "nonebot.adapters.feishu"
    # gewechat = "nonebot.adapters.gewechat"
    kritor = "nonebot.adapters.kritor"
    kook = "nonebot.adapters.kaiheila"
    mail = "nonebot.adapters.mail"
    minecraft = "nonebot.adapters.minecraft"
    mirai = "nonebot.adapters.mirai"
    milky = "nonebot.adapters.milky"
    onebot11 = "nonebot.adapters.onebot.v11"
    onebot12 = "nonebot.adapters.onebot.v12"
    qq = "nonebot.adapters.qq"
    satori = "nonebot.adapters.satori"
    telegram = "nonebot.adapters.telegram"
    yunhu = "nonebot.adapters.yunhu"
    # tail_chat = "nonebot_adapter_tailchat"  # TODO: Add tailchat support
    wxmp = "nonebot.adapters.wxmp"



from nonebot.plugin import PluginMetadata

from .config import Config
from .constraint import SupportAdapter as SupportAdapter
from .constraint import SupportAdapterModule
from .constraint import SupportScope as SupportScope
from .fetch import InfoFetcher as InfoFetcher
from .model import Member as Member
from .model import MuteInfo as MuteInfo
from .model import Role as Role
from .model import Scene as Scene
from .model import SceneType as SceneType
from .model import Session as Session
from .model import User as User
from .params import Interface as Interface
from .params import QryItrface as QryItrface
from .params import QueryInterface as QueryInterface
from .params import UniSession as UniSession
from .params import Uninfo as Uninfo
from .params import get_interface as get_interface
from .params import get_session as get_session
from .permission import ADMIN as ADMIN
from .permission import GROUP as GROUP
from .permission import GUILD as GUILD
from .permission import MEMBER as MEMBER
from .permission import OWNER as OWNER
from .permission import PRIVATE as PRIVATE
from .permission import ROLE_IN as ROLE_IN
from .permission import ROLE_LEVEL as ROLE_LEVEL
from .permission import ROLE_NOT_IN as ROLE_NOT_IN
from .permission import SCENE_IN as SCENE_IN
from .permission import SCENE_NOT_IN as SCENE_NOT_IN
from .permission import USER_IN as USER_IN
from .permission import USER_NOT_IN as USER_NOT_IN

__plugin_meta__ = PluginMetadata(
    name="通用信息",
    description="多平台的会话信息(用户、群组、频道)获取插件",
    usage="session_info: Uninfo",
    type="library",
    config=Config,
    homepage="https://github.com/RF-Tar-Railt/nonebot-plugin-uninfo",
    supported_adapters=set(SupportAdapterModule.__members__.values()),
)



"""init_db

迁移 ID: 14175fde8186
父迁移:
创建时间: 2024-10-30 16:27:05.032427

"""

from __future__ import annotations

from collections.abc import Sequence

from alembic import op
import sqlalchemy as sa

revision: str = "14175fde8186"
down_revision: str | Sequence[str] | None = None
branch_labels: str | Sequence[str] | None = ("nonebot_plugin_uninfo",)
depends_on: str | Sequence[str] | None = None


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "nonebot_plugin_uninfo_botmodel",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("self_id", sa.String(length=64), nullable=False),
        sa.Column("adapter", sa.String(length=32), nullable=False),
        sa.Column("scope", sa.String(length=32), nullable=False),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_nonebot_plugin_uninfo_botmodel")),
        sa.UniqueConstraint("self_id", "adapter", name="unique_bot"),
        info={"bind_key": "nonebot_plugin_uninfo"},
    )
    op.create_table(
        "nonebot_plugin_uninfo_scenemodel",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("bot_persist_id", sa.Integer(), nullable=False),
        sa.Column("parent_scene_persist_id", sa.Integer(), nullable=True),
        sa.Column("scene_id", sa.String(length=64), nullable=False),
        sa.Column("scene_type", sa.Integer(), nullable=False),
        sa.Column("scene_data", sa.JSON(), nullable=False),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_nonebot_plugin_uninfo_scenemodel")),
        sa.UniqueConstraint("bot_persist_id", "scene_id", "scene_type", name="unique_scene"),
        info={"bind_key": "nonebot_plugin_uninfo"},
    )
    op.create_table(
        "nonebot_plugin_uninfo_usermodel",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("bot_persist_id", sa.Integer(), nullable=False),
        sa.Column("user_id", sa.String(length=64), nullable=False),
        sa.Column("user_data", sa.JSON(), nullable=False),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_nonebot_plugin_uninfo_usermodel")),
        sa.UniqueConstraint("bot_persist_id", "user_id", name="unique_user"),
        info={"bind_key": "nonebot_plugin_uninfo"},
    )
    op.create_table(
        "nonebot_plugin_uninfo_sessionmodel",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("bot_persist_id", sa.Integer(), nullable=False),
        sa.Column("scene_persist_id", sa.Integer(), nullable=False),
        sa.Column("user_persist_id", sa.Integer(), nullable=False),
        sa.Column("member_data", sa.JSON(), nullable=True),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_nonebot_plugin_uninfo_sessionmodel")),
        sa.UniqueConstraint("bot_persist_id", "scene_persist_id", "user_persist_id", name="unique_session"),
        info={"bind_key": "nonebot_plugin_uninfo"},
    )
    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("nonebot_plugin_uninfo_botmodel")
    op.drop_table("nonebot_plugin_uninfo_scenemodel")
    op.drop_table("nonebot_plugin_uninfo_usermodel")
    op.drop_table("nonebot_plugin_uninfo_sessionmodel")
    # ### end Alembic commands ###



import importlib
import os
from pathlib import Path
from typing import cast
from warnings import warn

from nonebot import get_adapters

from ..fetch import InfoFetcher
from ..loader import BaseLoader

root = Path(__file__).parent
loaders: dict[str, BaseLoader] = {}
_adapters = [path.stem for path in root.iterdir() if path.is_dir() and not path.stem.startswith("_")]
for name in _adapters:
    try:
        module = importlib.import_module(f".{name}", __package__)
        loader = cast(BaseLoader, getattr(module, "Loader")())
        loaders[loader.get_adapter().value] = loader
    except Exception as e:
        warn(f"Failed to import uninfo adapter {name}: {e}", RuntimeWarning, 5)


INFO_FETCHER_MAPPING: dict[str, InfoFetcher] = {}
adapters = {}
try:
    adapters = get_adapters()
except Exception as e:
    warn(f"Failed to get nonebot adapters: {e}", RuntimeWarning, 5)

if os.environ.get("PLUGIN_UNINFO_TESTENV"):
    for adapter, loader in loaders.items():
        try:
            INFO_FETCHER_MAPPING[adapter] = loaders[adapter].get_fetcher()
        except Exception as e:
            warn(f"Failed to load uninfo adapter {adapter}: {e}", RuntimeWarning, 5)
elif not adapters:
    warn(
        "No adapters found, please make sure you have installed at least one adapter.",
        RuntimeWarning,
        5,
    )
else:
    for adapter in adapters:
        if adapter in loaders:
            try:
                INFO_FETCHER_MAPPING[adapter] = loaders[adapter].get_fetcher()
            except Exception as e:
                warn(f"Failed to load uninfo adapter {adapter}: {e}", RuntimeWarning, 5)
        else:
            warn(
                f"Adapter {adapter} is not found in the uninfo.adapters,"
                f"please go to the github repo and create an issue for it.",
                RuntimeWarning,
                5,
            )


def alter_get_fetcher(adapter_name: str):
    if adapter_name in loaders:
        try:
            INFO_FETCHER_MAPPING[adapter_name] = loaders[adapter_name].get_fetcher()
            return INFO_FETCHER_MAPPING[adapter_name]
        except Exception as e:
            warn(f"Failed to load uninfo adapter {adapter_name}: {e}", RuntimeWarning, 6)
            return None
    warn(
        f"Adapter {adapter_name} is not found in the uninfo.adapters,"
        f"please go to the github repo and create an issue for it.",
        RuntimeWarning,
        6,
    )
    return None



from pathlib import Path

from nonebot.adapters.telegram import Bot
from nonebot.adapters.telegram.event import (
    CallbackQueryEvent,
    ForumTopicEditedMessageEvent,
    ForumTopicMessageEvent,
    GroupEditedMessageEvent,
    GroupMessageEvent,
    LeftChatMemberEvent,
    NewChatMemberEvent,
    PrivateEditedMessageEvent,
    PrivateMessageEvent,
)
from nonebot.adapters.telegram.model import User as TelegramUser
from nonebot.exception import ActionFailed

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Role, Scene, SceneType, User

ROLES = {
    "creator": ("OWNER", 100),
    "administrator": ("ADMINISTRATOR", 10),
    "member": ("MEMBER", 1),
    "restricted": ("RESTRICTED", 0),
    "left": ("LEFT", 0),
    "kicked": ("KICKED", 0),
}


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            nick=data["nickname"],
            avatar=data.get("avatar"),
        )

    def extract_scene(self, data):
        if "chat_id" in data:
            if "thread_id" in data:
                return Scene(
                    id=data["thread_id"],
                    type=SceneType.CHANNEL_TEXT,
                    name=data["chat_name"],
                    parent=Scene(
                        id=data["chat_id"],
                        type=SceneType.GUILD,
                        name=data["chat_name"],
                    ),
                )
            return Scene(
                id=data["chat_id"],
                type=SceneType.GROUP,
                name=data["chat_name"],
            )
        return Scene(
            id=data["user_id"],
            name=data["name"],
            type=SceneType.PRIVATE,
            avatar=data.get("avatar"),
        )

    def extract_member(self, data, user: User | None):
        if "chat_id" not in data:
            return None
        if user:
            return Member(
                user=user,
                nick=data["nickname"],
                roles=[Role(*ROLES[data["role"]], data["role"])] if "role" in data else [],
            )
        return Member(
            User(
                id=data["user_id"],
                name=data["name"],
                nick=data["nickname"],
                avatar=data.get("avatar"),
            ),
            nick=data["nickname"],
            roles=[Role(*ROLES[data["role"]], data["role"])] if "role" in data else [],
        )

    async def query_user(self, bot: Bot, user_id: str):
        data = await _supply_userdata(bot, user_id)
        if data["name"] or data["nickname"] or data["avatar"]:
            return self.extract_user(data)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            if user := await self.query_user(bot, scene_id):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "nickname": user.nick,
                    "avatar": user.avatar,
                }
                return self.extract_scene(data)

        elif scene_type == SceneType.GROUP:
            chat = await bot.get_chat(chat_id=int(scene_id))
            data = {
                "chat_id": str(chat.id),
                "chat_name": chat.title,
            }
            return self.extract_scene(data)

        elif scene_type >= SceneType.CHANNEL_TEXT and parent_scene_id is not None:
            chat = await bot.get_chat(chat_id=parent_scene_id)
            data = {
                "chat_id": str(chat.id),
                "chat_name": chat.title,
                "thread_id": scene_id,
            }
            return self.extract_scene(data)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type >= SceneType.GROUP:
            member = await bot.get_chat_member(chat_id=parent_scene_id, user_id=int(user_id))
            data = await _supply_userdata(bot, member.user)
            data["role"] = member.status
            return self.extract_member(data, None)

    def query_users(self, bot: Bot):
        raise NotImplementedError

    def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        raise NotImplementedError

    def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        raise NotImplementedError

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.telegram,
            "scope": SupportScope.telegram,
        }


fetcher = InfoFetcher(SupportAdapter.telegram)


async def _supply_userdata(bot: Bot, user: str | TelegramUser):

    if isinstance(user, TelegramUser):
        if str(user.id) == str(bot.self_id):
            user = await bot.get_me()
        res = {
            "user_id": str(user.id),
            "name": user.username or "",
            "nickname": user.first_name + (f" {user.last_name}" if user.last_name else ""),
            "avatar": None,
        }
    elif user == str(bot.self_id):
        _user = await bot.get_me()
        res = {
            "user_id": str(_user.id),
            "name": _user.username or "",
            "nickname": _user.first_name + (f" {_user.last_name}" if _user.last_name else ""),
            "avatar": None,
        }
    else:
        res = {
            "user_id": user,
            "name": "",
            "nickname": "",
            "avatar": None,
        }
    try:
        if isinstance(user, str):
            if user == bot.self_id:
                _user = await bot.get_me()
            else:
                _user = await bot.get_chat(chat_id=int(user))
            res["name"] = _user.username or ""
            nickname = _user.first_name
            if nickname and _user.last_name:
                nickname += f" {_user.last_name}"
            res["nickname"] = nickname or ""
        else:
            _user = user
        profile_photos = await bot.get_user_profile_photos(user_id=_user.id, limit=1)
        if profile_photos.total_count > 0:
            file = await bot.get_file(file_id=profile_photos.photos[0][-1].file_id)
            if file.file_path:
                if Path(file.file_path).exists():
                    # 本地搭建的 Telegram Bot API 会传给你本地的文件路径
                    res["avatar"] = Path(file.file_path).as_uri()
                else:
                    res["avatar"] = f"https://api.telegram.org/file/bot{bot.bot_config.token}/{file.file_path}"
    except ActionFailed:
        pass
    return res


@fetcher.supply
async def _(bot: Bot, event: PrivateMessageEvent | PrivateEditedMessageEvent):
    return await _supply_userdata(bot, event.from_)


@fetcher.supply
async def _(bot: Bot, event: GroupMessageEvent | GroupEditedMessageEvent):
    base = await _supply_userdata(bot, event.from_)
    base["chat_id"] = str(event.chat.id)
    base["chat_name"] = event.chat.title
    try:
        member = await bot.get_chat_member(chat_id=event.chat.id, user_id=event.from_.id)
        base["role"] = member.status
    except ActionFailed:
        pass
    return base


@fetcher.supply
async def _(bot: Bot, event: ForumTopicMessageEvent | ForumTopicEditedMessageEvent):
    base = await _supply_userdata(bot, event.from_)
    base["chat_id"] = str(event.chat.id)
    base["chat_name"] = event.chat.title
    base["thread_id"] = str(event.message_thread_id)
    try:
        member = await bot.get_chat_member(chat_id=event.chat.id, user_id=event.from_.id)
        base["role"] = member.status
    except ActionFailed:
        pass
    return base


@fetcher.supply
async def _(bot: Bot, event: LeftChatMemberEvent):
    base = await _supply_userdata(bot, event.left_chat_member)
    base["chat_id"] = str(event.chat.id)
    base["chat_name"] = event.chat.title
    try:
        member = await bot.get_chat_member(chat_id=event.chat.id, user_id=event.left_chat_member.id)
        base["role"] = member.status
    except ActionFailed:
        pass
    return base


@fetcher.supply
async def _(bot: Bot, event: NewChatMemberEvent):
    base = await _supply_userdata(bot, event.new_chat_members[0])
    base["chat_id"] = str(event.chat.id)
    base["chat_name"] = event.chat.title
    try:
        member = await bot.get_chat_member(chat_id=event.chat.id, user_id=event.new_chat_members[0].id)
        base["role"] = member.status
    except ActionFailed:
        pass
    return base


@fetcher.supply
async def _(bot: Bot, event: CallbackQueryEvent):
    base = await _supply_userdata(bot, event.from_)
    if event.message:
        base["chat_id"] = str(event.message.chat.id)
        base["chat_name"] = event.message.chat.title
        try:
            member = await bot.get_chat_member(chat_id=event.message.chat.id, user_id=event.from_.id)
            base["role"] = member.status
        except ActionFailed:
            pass
    return base



from nonebot.adapters.wxmp import Bot
from nonebot.adapters.wxmp.event import Event

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Scene, SceneType, User


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["user_id"],
        )

    def extract_scene(self, data):
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
        )

    def extract_member(self, data, user: User | None):
        return None

    async def query_user(self, bot: Bot, user_id: str):
        raise NotImplementedError

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        raise NotImplementedError

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        raise NotImplementedError

    def query_users(self, bot: Bot):
        raise NotImplementedError

    def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        raise NotImplementedError

    def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        raise NotImplementedError

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.wxmp,
            "scope": SupportScope.wechat_oap,
        }


fetcher = InfoFetcher(SupportAdapter.wxmp)


@fetcher.supply
async def _(bot: Bot, event: Event):
    return {
        "user_id": event.user_id,
    }



from typing import Any

from nonebot.adapters.feishu import Bot
from nonebot.adapters.feishu.event import GroupMessageEvent, PrivateMessageEvent
from nonebot.exception import ActionFailed

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Role, Scene, SceneType, User


def _handle_gender(gender: int | None) -> str:
    return "male" if gender == 1 else "female" if gender == 2 else "unknown"


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data: dict[str, Any]) -> User:
        return User(
            id=data["user_id"],
            name=data["name"],
            nick=data.get("nickname"),
            avatar=data.get("avatar"),
            gender=data.get("gender", "unknown"),
        )

    def extract_scene(self, data: dict[str, Any]) -> Scene:
        if "group_id" in data:
            return Scene(
                id=data["group_id"],
                type=SceneType.GROUP,
                name=data.get("group_name"),
                avatar=data.get("group_avatar"),
            )
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
            name=data["name"],
            avatar=data.get("avatar"),
        )

    def extract_member(self, data: dict[str, Any], user: User | None):
        if "group_id" not in data:
            return None
        if user:
            return Member(
                user=user,
                nick=data.get("member_name"),
                roles=[data["role"]] if "role" in data else [Role("MEMBER", 1, "member")],
            )
        return Member(
            User(
                id=data["user_id"],
                name=data["name"],
                nick=data.get("nickname"),
                avatar=data.get("avatar"),
            ),
            nick=data.get("member_name"),
            roles=[data["role"]] if "role" in data else [Role("MEMBER", 1, "member")],
        )

    async def query_user(self, bot: Bot, user_id: str):

        resp = await bot.call_api(
            f"contact/v3/users/{user_id}",
            method="GET",
            query={"user_id_type": "open_id"},
        )
        info = resp["data"]["user"]
        return self.extract_user(
            {
                "user_id": info["open_id"],
                "name": info.get("name"),
                "nickname": info.get("nickname"),
                "avatar": info.get("avatar", {}).get("avatar_origin"),
                "gender": _handle_gender(info.get("gender")),
            }
        )

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            if user := await self.query_user(bot, scene_id):
                return self.extract_scene(
                    {
                        "user_id": user.id,
                        "name": user.name,
                        "avatar": user.avatar,
                    }
                )

        elif scene_type == SceneType.GROUP:
            resp = await bot.call_api(
                f"im/v1/chats/{scene_id}",
                method="GET",
            )
            info = resp["data"]
            return self.extract_scene(
                {
                    "group_id": scene_id,
                    "group_name": info.get("name"),
                    "group_avatar": info.get("avatar"),
                }
            )

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        raise NotImplementedError

    async def query_users(self, bot: Bot):
        has_more1 = True
        has_more2 = True
        params1 = {}
        params2 = {}
        while has_more1:
            resp1 = await bot.call_api(
                "contact/v3/users/group/simplelist",
                method="GET",
                query=params1,
            )
            params1["page_token"] = resp1["data"]["page_token"]
            has_more1 = resp1["data"]["has_more"]
            for user_group in resp1["data"]["groupList"]:
                while has_more2:
                    resp2 = await bot.call_api(
                        f"contact/v3/users/simplelist/{user_group['id']}/member/simplelist",
                        method="GET",
                        query=params2,
                    )
                    params2["page_token"] = resp2["data"]["page_token"]
                    has_more2 = resp2["data"]["has_more"]
                    for user in resp2["data"]["memberlist"]:
                        member_id = user["user_id"]
                        resp = await bot.call_api(
                            f"contact/v3/users/{member_id}",
                            method="GET",
                            query={"user_id_type": user["member_id_type"]},
                        )
                        info = resp["data"]["user"]
                        gender = _handle_gender(info.get("gender"))
                        yield self.extract_user(
                            {
                                "user_id": info["open_id"],
                                "name": info.get("name"),
                                "avatar": info.get("avatar", {}).get("avatar_origin"),
                                "nickname": info.get("nickname"),
                                "gender": gender,
                            }
                        )

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is None or scene_type == SceneType.PRIVATE:
            async for user in self.query_users(bot):
                yield self.extract_scene(
                    {
                        "user_id": user.id,
                        "name": user.name,
                        "avatar": user.avatar,
                    }
                )
            if scene_type == SceneType.PRIVATE:
                return

        if scene_type is not None and scene_type != SceneType.GROUP:
            return

        params = {}
        has_more = True
        while has_more:
            resp = await bot.call_api(
                "im/v1/chats/",
                method="GET",
                query=params,
            )
            page_token = resp["data"]["page_token"]
            params["page_token"] = page_token
            has_more = resp["data"]["has_more"]
            for chat in resp["data"]["items"]:
                yield self.extract_scene(
                    {
                        "group_id": chat["chat_id"],
                        "group_name": chat["name"],
                        "group_avatar": chat["avatar"],
                    }
                )

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        if scene_type != SceneType.GROUP:
            return
        group_id = parent_scene_id

        resp = await bot.call_api(
            f"im/v1/chats/{group_id}",
            method="GET",
        )
        owner_id = resp["data"]["owner_id"]
        owner_id_type = resp["data"]["owner_id_type"]
        resp = await bot.call_api(
            f"contact/v3/users/{owner_id}",
            method="GET",
            query={"user_id_type": owner_id_type},
        )
        owner_id = resp["data"]["user"]["open_id"]
        has_more = True
        params = {}
        while has_more:
            resp = await bot.call_api(
                f"im/v1/chats/{group_id}/members",
                method="GET",
                query=params,
            )
            page_token = resp["data"]["page_token"]
            params["page_token"] = page_token
            has_more = resp["data"]["has_more"]
            for member in resp["data"]["items"]:
                resp = await bot.call_api(
                    f"contact/v3/users/{member['member_id']}",
                    method="GET",
                    query={"user_id_type": member["member_id_type"]},
                )
                info = resp["data"]["user"]
                yield self.extract_member(
                    {
                        "user_id": info["open_id"],
                        "name": info.get("name"),
                        "avatar": info.get("avatar", {}).get("avatar_origin"),
                        "nickname": info.get("nickname"),
                        "member_name": member["name"],
                        "group_id": group_id,
                        "role": (
                            Role("OWNER", 100, "owner")
                            if member["open_id"] == owner_id
                            else Role("MEMBER", 1, "member")
                        ),
                    },
                    None,
                )

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.feishu,
            "scope": SupportScope.feishu,
        }


fetcher = InfoFetcher(SupportAdapter.feishu)


@fetcher.supply
async def _(bot: Bot, event: GroupMessageEvent | PrivateMessageEvent):
    user = {}
    try:
        resp = await bot.call_api(
            f"contact/v3/users/{event.event.sender.sender_id.open_id}",
            method="GET",
            query={"user_id_type": "open_id"},
        )
        info = resp["data"]["user"]
        user["gender"] = _handle_gender(info.get("gender"))
        user["name"] = info.get("name")
        user["avatar"] = info.get("avatar", {}).get("avatar_origin")
        user["nickname"] = info.get("nickname")
    except ActionFailed:
        pass
    base = {
        "user_id": event.event.sender.sender_id.open_id,
        "name": "",
        **user,
    }
    if isinstance(event, GroupMessageEvent):
        chat_id = event.event.message.chat_id
        base["group_id"] = chat_id
        try:
            resp = await bot.call_api(
                f"im/v1/chats/{chat_id}",
                method="GET",
            )
            base["group_name"] = resp["data"]["name"]
            base["group_avatar"] = resp["data"]["avatar"]
            resp1 = await bot.call_api(
                f"contact/v3/users/{resp['data']['owner_id']}",
                method="GET",
                query={"user_id_type": resp["data"]["owner_id_type"]},
            )
            if event.event.sender.sender_id.open_id == resp1["data"]["user"]["open_id"]:
                base["role"] = Role("OWNER", 100, "owner")
        except ActionFailed:
            pass
    return base



from nonebot.adapters.dodo import Bot
from nonebot.adapters.dodo.event import (
    CardMessageButtonClickEvent,
    CardMessageFormSubmitEvent,
    CardMessageListSubmitEvent,
    ChannelArticleCommentEvent,
    ChannelArticleEvent,
    ChannelMessageEvent,
    ChannelVoiceMemberJoinEvent,
    ChannelVoiceMemberLeaveEvent,
    MessageReactionEvent,
    PersonalMessageEvent,
)
from nonebot.adapters.dodo.models import ChannelType

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Role, Scene, SceneType, User


def _handle_gender(sex: int) -> str:
    return "male" if sex == 1 else "female" if sex == 0 else "unknown"


async def _handle_roles(bot: Bot, guild_id: str, user_id: str):
    res = []
    resp = await bot.get_member_role_list(island_source_id=guild_id, dodo_source_id=user_id)
    for role in resp:
        perm = int(role.permission)  # type: ignore
        if perm & (1 << 0) == (1 << 0):
            if perm & (1 << 1) == (1 << 1):
                res.append(("OWNER", 100, role.role_name))
            res.append(("ADMINISTRATOR", 10, role.role_name))
        if perm & (1 << 5) == (1 << 5):
            res.append(("CHANNEL_ADMINISTRATOR", 9, role.role_name))
        res.append((str(role.role_id), 1, role.role_name))
    if not res:
        return [Role("MEMBER", 1, "member")]
    return [Role(*r) for r in res]


CHANNEL_TYPE = {
    ChannelType.TEXT: SceneType.CHANNEL_TEXT,
    ChannelType.VOICE: SceneType.CHANNEL_VOICE,
    ChannelType.ARTICLE: SceneType.CHANNEL_TEXT,
    ChannelType.LINK: SceneType.CHANNEL_CATEGORY,
    ChannelType.PROFILE: SceneType.CHANNEL_CATEGORY,
}


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            avatar=data["avatar"],
            gender=data["gender"],
        )

    def extract_scene(self, data):
        if "guild_id" in data:
            if "channel_id" in data:
                return Scene(
                    id=data["channel_id"],
                    name=data.get("channel_name"),
                    type=data.get("channel_type", SceneType.CHANNEL_TEXT),
                    parent=Scene(
                        id=data["guild_id"],
                        name=data.get("guild_name"),
                        type=SceneType.GUILD,
                        avatar=data.get("guild_avatar"),
                    ),
                )
            return Scene(
                id=data["guild_id"],
                name=data.get("guild_name"),
                type=SceneType.GUILD,
                avatar=data.get("guild_avatar"),
            )
        if "channel_id" in data:
            return Scene(
                id=data["channel_id"],
                name=data.get("channel_name"),
                type=data.get("channel_type", SceneType.CHANNEL_TEXT),
            )
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
            name=data["name"],
            avatar=data["avatar"],
        )

    def extract_member(self, data, user: User | None):
        if "guild_id" in data or "channel_id" in data:
            if user:
                return Member(user, nick=data["nickname"], roles=data.get("roles", []), joined_at=data.get("joined_at"))
            return Member(
                User(
                    id=data["user_id"],
                    name=data["name"],
                    avatar=data.get("avatar"),
                    gender=data["gender"],
                ),
                nick=data["nickname"],
                roles=data.get("roles", []),
                joined_at=data.get("joined_at"),
            )
        return None

    async def query_user(self, bot: Bot, user_id: str):
        if user_id == bot.self_id:
            info = await bot.get_bot_info()
            return User(
                id=info.dodo_source_id,
                name=info.nick_name,
                avatar=info.avatar_url,
            )
        raise NotImplementedError

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.GUILD:
            guild = await bot.get_island_info(island_source_id=scene_id)
            return self.extract_scene(
                {
                    "guild_id": guild.island_source_id,
                    "guild_name": guild.island_name,
                    "guild_avatar": guild.cover_url,
                }
            )

        elif scene_type >= SceneType.CHANNEL_TEXT:
            channel = await bot.get_channel_info(channel_id=scene_id)
            return self.extract_scene(
                {
                    "channel_id": channel.channel_id,
                    "channel_name": channel.channel_name,
                    "channel_type": CHANNEL_TYPE.get(channel.channel_type, SceneType.CHANNEL_TEXT),
                }
            )

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        guild_id = parent_scene_id

        member = await bot.get_member_info(island_source_id=guild_id, dodo_source_id=user_id)
        user = User(
            id=member.dodo_source_id,
            name=member.personal_nick_name,
            avatar=member.avatar_url,
            gender=_handle_gender(member.sex),
        )
        return Member(
            user=user,
            nick=member.nick_name,
            roles=await _handle_roles(bot, guild_id, member.dodo_source_id),
            joined_at=member.join_time,
        )

    def query_users(self, bot: Bot):
        raise NotImplementedError

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type in (SceneType.PRIVATE, SceneType.GROUP):
            return

        guilds = await bot.get_island_list()
        for guild in guilds:
            if parent_scene_id is None or parent_scene_id == guild.island_source_id:
                _guild = Scene(
                    id=guild.island_source_id,
                    type=SceneType.GUILD,
                    name=guild.island_name,
                    avatar=guild.cover_url,
                )
                if scene_type is None or scene_type == SceneType.GUILD:
                    yield _guild
                if scene_type == SceneType.GUILD:
                    continue
                channels = await bot.get_channel_list(island_source_id=guild.island_source_id)
                for channel in channels:
                    yield Scene(
                        id=channel.channel_id,
                        type=CHANNEL_TYPE.get(channel.channel_type, SceneType.CHANNEL_TEXT),
                        name=channel.channel_name,
                        parent=_guild,
                    )

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        guild_id = parent_scene_id

        members = await bot.get_member_list(island_source_id=guild_id, page_size=100)
        while members.list:
            for member in members.list:
                user = User(
                    id=member.dodo_source_id,
                    name=member.personal_nick_name,
                    avatar=member.avatar_url,
                    gender=_handle_gender(member.sex),
                )
                yield Member(
                    user=user,
                    nick=member.nick_name,
                    roles=await _handle_roles(bot, guild_id, member.dodo_source_id),
                    joined_at=member.join_time,
                )
            if len(members.list) < 100:
                break
            members = await bot.get_member_list(island_source_id=guild_id, page_size=100, max_id=members.max_id)

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.dodo,
            "scope": SupportScope.dodo,
        }


fetcher = InfoFetcher(SupportAdapter.dodo)


@fetcher.supply
async def _(bot: Bot, event: PersonalMessageEvent):
    return {
        "user_id": event.dodo_source_id,
        "name": event.personal.nick_name,
        "avatar": event.personal.avatar_url,
        "gender": _handle_gender(event.personal.sex),
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: (
        ChannelMessageEvent
        | MessageReactionEvent
        | CardMessageButtonClickEvent
        | CardMessageFormSubmitEvent
        | CardMessageListSubmitEvent
        | ChannelVoiceMemberJoinEvent
        | ChannelVoiceMemberLeaveEvent
        | ChannelArticleEvent
        | ChannelArticleCommentEvent
    ),
):
    base = {
        "user_id": event.dodo_source_id,
        "name": event.personal.nick_name,
        "avatar": event.personal.avatar_url,
        "gender": _handle_gender(event.personal.sex),
    }
    guild = await bot.get_island_info(island_source_id=event.island_source_id)
    base |= {
        "guild_id": event.island_source_id,
        "guild_name": guild.island_name,
        "guild_avatar": guild.cover_url,
    }
    channel = await bot.get_channel_info(channel_id=event.channel_id)
    base |= {
        "channel_id": event.channel_id,
        "channel_name": channel.channel_name,
        "channel_type": CHANNEL_TYPE.get(channel.channel_type, SceneType.CHANNEL_TEXT),
    }
    base |= {
        "nickname": event.member.nick_name,
        "roles": await _handle_roles(bot, event.island_source_id, event.dodo_source_id),
        "joined_at": event.member.join_time,
    }
    return base



from datetime import datetime, timedelta

from nonebot.adapters.kaiheila import Bot
from nonebot.adapters.kaiheila.api.model import Channel as KookChannel
from nonebot.adapters.kaiheila.event import Event, HeartbeatMetaEvent, LifecycleMetaEvent

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, MuteInfo, Role, Scene, SceneType, User


async def _handle_roles(bot: Bot, guild_id: str, roles: list[int]):
    if not roles:
        return [Role("MEMBER", 1, "member")]
    res = []
    resp = await bot.guildRole_list(guild_id=guild_id)
    if not resp.roles:
        return [Role("MEMBER", 1, "member")]
    for role in resp.roles:
        if role.role_id not in roles:
            continue
        perm = int(role.permissions)  # type: ignore
        if perm & (1 << 0) == (1 << 0):
            if perm & (1 << 1) == (1 << 1):
                res.append(("OWNER", 100, role.name))
            res.append(("ADMINISTRATOR", 10, role.name))
        if perm & (1 << 5) == (1 << 5):
            res.append(("CHANNEL_ADMINISTRATOR", 9, role.name))
        res.append((str(role.role_id), 1, role.name))
    if not res:
        return [Role("MEMBER", 1, "member")]
    return [Role(*item) for item in res]


def _handle_channel_type(channel: KookChannel):
    if channel.is_category:
        return SceneType.CHANNEL_CATEGORY
    if channel.type == 2:
        return SceneType.CHANNEL_VOICE
    return SceneType.CHANNEL_TEXT


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            avatar=data["avatar"],
        )

    def extract_scene(self, data):
        if "guild_id" in data:
            if "channel_id" in data:
                return Scene(
                    id=data["channel_id"],
                    name=data.get("channel_name"),
                    type=data.get("channel_type", SceneType.CHANNEL_TEXT),
                    parent=Scene(
                        id=data["guild_id"],
                        name=data.get("guild_name"),
                        type=SceneType.GUILD,
                        avatar=data.get("guild_avatar"),
                    ),
                )
            return Scene(
                id=data["guild_id"],
                name=data.get("guild_name"),
                type=SceneType.GUILD,
                avatar=data.get("guild_avatar"),
            )
        if "channel_id" in data:
            return Scene(
                id=data["channel_id"],
                name=data.get("channel_name"),
                type=data.get("channel_type", SceneType.CHANNEL_TEXT),
            )
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
            name=data["name"],
            avatar=data["avatar"],
        )

    def extract_member(self, data, user: User | None):
        if "guild_id" in data or "channel_id" in data:
            if user:
                return Member(user, nick=data["nickname"], roles=data.get("roles", []), joined_at=data.get("joined_at"))
            return Member(
                User(
                    id=data["user_id"],
                    name=data["name"],
                    avatar=data.get("avatar"),
                ),
                nick=data["nickname"],
                roles=data.get("roles", []),
                joined_at=data.get("joined_at"),
            )
        return None

    async def query_user(self, bot: Bot, user_id: str):
        if bot.self_id == user_id:
            info = await bot.user_me()
        else:
            info = await bot.user_view(user_id=user_id)
        return User(
            id=str(info.id_),
            name=info.username,
            avatar=info.avatar,
        )

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.GUILD:
            guild = await bot.guild_view(guild_id=scene_id)
            return self.extract_scene(
                {
                    "guild_id": guild.id_,
                    "guild_name": guild.name,
                    "guild_avatar": guild.icon,
                }
            )

        elif scene_type >= SceneType.CHANNEL_TEXT:
            channel = await bot.channel_view(target_id=scene_id)
            return self.extract_scene(
                {
                    "channel_id": channel.id_,
                    "channel_name": channel.name,
                    "channel_type": _handle_channel_type(channel),
                }
            )

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type != SceneType.GUILD:
            return
        guild_id = parent_scene_id

        member = await bot.user_view(guild_id=guild_id, user_id=user_id)
        user = User(
            id=str(member.id_),
            name=member.username,
            avatar=member.avatar,
        )
        return Member(
            user=user,
            nick=member.nickname or member.username,
            roles=await _handle_roles(bot, guild_id, member.roles or []),
            joined_at=datetime.fromtimestamp(member.joined_at / 1000) if member.joined_at else None,
            mute=MuteInfo(muted=True, duration=timedelta(60)) if member.status == 10 else None,
        )

    async def query_users(self, bot: Bot):
        resp = await bot.userChat_list()
        while True:
            for chat in resp.user_chats or []:
                if chat.target_info:
                    yield User(
                        id=chat.target_info.id_ or "",
                        name=chat.target_info.username,
                        avatar=chat.target_info.avatar,
                    )
            if not resp.meta or resp.meta.page == resp.meta.page_total:
                break
            resp = await bot.userChat_list(page=(resp.meta.page or 0) + 1)

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.GROUP:
            return

        if scene_type is None or scene_type == SceneType.PRIVATE:
            async for user in self.query_users(bot):
                yield self.extract_scene(
                    {
                        "user_id": user.id,
                        "name": user.name,
                        "avatar": user.avatar,
                    }
                )
            if scene_type == SceneType.PRIVATE:
                return

        resp = await bot.guild_list()
        while True:
            for guild in resp.guilds or []:
                if not guild.id_:
                    continue
                if parent_scene_id is None or str(guild.id_) == parent_scene_id:
                    _guild = Scene(
                        id=str(guild.id_),
                        type=SceneType.GUILD,
                        name=guild.name,
                        avatar=guild.icon,
                    )
                    if scene_type is None or scene_type == SceneType.GUILD:
                        yield _guild
                    if scene_type == SceneType.GUILD:
                        continue
                    channels = guild.channels or (await bot.channel_list(guild_id=guild.id_)).channels or []
                    for channel in channels:
                        yield Scene(
                            id=str(channel.id_),
                            type=_handle_channel_type(channel),
                            name=channel.name,
                            parent=_guild,
                        )
            if not resp.meta or resp.meta.page == resp.meta.page_total:
                break
            resp = await bot.guild_list(page=(resp.meta.page or 0) + 1)

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        if scene_type in (SceneType.PRIVATE, SceneType.GROUP):
            return
        if scene_type == SceneType.GUILD:
            resp = await bot.guild_userList(guild_id=parent_scene_id)
            while True:
                for member in resp.users or []:
                    user = User(
                        id=str(member.id_),
                        name=member.username,
                        avatar=member.avatar,
                    )
                    yield Member(
                        user=user,
                        nick=member.nickname or member.username,
                        roles=await _handle_roles(bot, parent_scene_id, member.roles or []),
                        joined_at=datetime.fromtimestamp(member.joined_at / 1000) if member.joined_at else None,
                        mute=MuteInfo(muted=True, duration=timedelta(60)) if member.status == 10 else None,
                    )
                if not resp.meta or resp.meta.page == resp.meta.page_total:
                    break
                resp = await bot.guild_userList(guild_id=parent_scene_id, page=(resp.meta.page or 0) + 1)
        else:
            users = await bot.channel_userList(channel_id=parent_scene_id)
            for member in users:
                user = User(
                    id=str(member.id_),
                    name=member.username,
                    avatar=member.avatar,
                )
                yield Member(
                    user=user,
                    nick=member.nickname or member.username,
                    roles=await _handle_roles(bot, parent_scene_id, member.roles or []),
                    joined_at=datetime.fromtimestamp(member.joined_at / 1000) if member.joined_at else None,
                    mute=MuteInfo(muted=True, duration=timedelta(60)) if member.status == 10 else None,
                )

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.kook,
            "scope": SupportScope.kook,
        }


fetcher = InfoFetcher(SupportAdapter.kook)


@fetcher.supply
async def _(bot: Bot, event: LifecycleMetaEvent):
    raise NotImplementedError


@fetcher.supply
async def _(bot: Bot, event: HeartbeatMetaEvent):
    raise NotImplementedError


@fetcher.supply_wildcard
async def _(bot: Bot, event: Event):
    if event.user_id == "SYSTEM":
        base = {
            "user_id": bot.self_id,
            "name": bot.self_name,
            "nickname": bot.self_name,
            "avatar": "",
        }
    else:
        user = event.extra.author or await bot.user_view(user_id=event.user_id)
        base = {
            "user_id": user.id_,
            "name": user.username,
            "nickname": user.nickname,
            "avatar": user.avatar,
        }
    if event.channel_type == "PERSON":
        return base
    if event.type_ != 255:
        channel = await bot.channel_view(target_id=event.target_id)
        base |= {
            "channel_id": channel.id_,
            "channel_name": channel.name or event.extra.channel_name,
            "channel_type": _handle_channel_type(channel),
        }
        if channel.guild_id or event.extra.guild_id:
            guild_id = channel.guild_id or event.extra.guild_id or ""
            guild = await bot.guild_view(guild_id=guild_id)
            base |= {
                "guild_id": guild.id_,
                "guild_name": guild.name,
                "guild_avatar": guild.icon,
            }
            member = await bot.user_view(guild_id=guild_id, user_id=event.user_id)
            base |= {
                "nickname": member.nickname,
                "roles": await _handle_roles(bot, guild_id, member.roles or []),
                "joined_at": datetime.fromtimestamp(member.joined_at / 1000) if member.joined_at else None,
            }
    else:
        guild_id = event.extra.guild_id or event.target_id
        guild = await bot.guild_view(guild_id=guild_id)
        base |= {
            "guild_id": guild.id_,
            "guild_name": guild.name,
            "guild_avatar": guild.icon,
        }
        member = await bot.user_view(guild_id=guild_id, user_id=event.user_id)
        base |= {
            "nickname": member.nickname,
            "roles": await _handle_roles(bot, guild_id, member.roles or []),
            "joined_at": datetime.fromtimestamp(member.joined_at / 1000) if member.joined_at else None,
        }
    return base



from datetime import datetime, timedelta

from nonebot.adapters.mirai import Bot
from nonebot.adapters.mirai.event import (
    BotInvitedJoinGroupRequestEvent,
    BotJoinGroupEvent,
    BotLeaveEventDisband,
    BotLeaveEventKick,
    BotMuteEvent,
    BotUnmuteEvent,
    FriendAddEvent,
    FriendDeleteEvent,
    FriendInputStatusChangedEvent,
    FriendMessage,
    FriendNickChangedEvent,
    FriendRecallEvent,
    GroupAllowMemberInviteEvent,
    GroupEntranceAnnouncementChangeEvent,
    GroupMessage,
    GroupMuteAllEvent,
    GroupNameChangeEvent,
    GroupRecallEvent,
    MemberCardChangeEvent,
    MemberHonorChangeEvent,
    MemberJoinEvent,
    MemberJoinRequestEvent,
    MemberLeaveEventKick,
    MemberLeaveEventQuit,
    MemberMuteEvent,
    MemberPermissionChangeEvent,
    MemberSpecialTitleChangeEvent,
    MemberUnmuteEvent,
    NewFriendRequestEvent,
    NudgeEvent,
    StrangerMessage,
    TempMessage,
)
from nonebot.adapters.mirai.model.relationship import Group, MemberPerm
from nonebot.exception import ActionFailed
from nonebot.internal.adapter import Event

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, MuteInfo, Role, Scene, SceneType, User

ROLES = {
    MemberPerm.Owner: Role("OWNER", 100, "OWNER"),
    MemberPerm.Administrator: Role("ADMINISTRATOR", 10, "ADMINISTRATOR"),
    MemberPerm.Member: Role("MEMBER", 1, "MEMBER"),
}


class InfoFetcher(BaseInfoFetcher):
    def get_session_id(self, event: Event) -> str:
        if isinstance(event, (BotJoinGroupEvent, MemberJoinEvent)) and event.inviter:
            return f"{event.get_session_id()}_{event.inviter.id}"
        if (
            isinstance(
                event,
                (
                    BotMuteEvent,
                    BotUnmuteEvent,
                    BotLeaveEventKick,
                    BotLeaveEventDisband,
                    GroupNameChangeEvent,
                    GroupEntranceAnnouncementChangeEvent,
                    GroupMuteAllEvent,
                    GroupAllowMemberInviteEvent,
                    MemberLeaveEventKick,
                    MemberCardChangeEvent,
                    MemberMuteEvent,
                    MemberUnmuteEvent,
                ),
            )
            and event.operator
        ):
            return f"{event.get_session_id()}_{event.operator.id}"
        if isinstance(event, GroupRecallEvent):
            return f"{event.get_session_id()}_{event.operator}"
        if isinstance(event, NudgeEvent):
            return f"{event.get_session_id()}_{event.supplicant}"
        return event.get_session_id()

    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            nick=data.get("nickname"),
            avatar=f"http://q1.qlogo.cn/g?b=qq&nk={data['user_id']}&s=640",
            gender=data.get("gender", "unknown"),
        )

    def extract_scene(self, data):
        if "group_id" not in data:
            return Scene(
                id=data["user_id"],
                type=SceneType.PRIVATE,
            )
        return Scene(
            id=data["group_id"],
            type=SceneType.GROUP,
            name=data["group_name"],
            avatar=f"https://p.qlogo.cn/gh/{data['group_id']}/{data['group_id']}/",
        )

    def extract_member(self, data, user: User | None):
        if "group_id" not in data:
            return None
        if user:
            return Member(
                user=user,
                nick=data["card"],
                roles=[ROLES[_role]] if (_role := data.get("role")) else [],
                joined_at=datetime.fromtimestamp(data["join_time"]) if data["join_time"] else None,
                mute=(
                    MuteInfo(muted=True, duration=timedelta(seconds=data["mute_duration"]))
                    if "mute_duration" in data
                    else None
                ),
            )
        return Member(
            User(
                id=data["user_id"],
                name=data["name"],
                nick=data.get("nickname"),
                avatar=f"https://q2.qlogo.cn/headimg_dl?dst_uin={data['user_id']}&spec=640",
            ),
            nick=data["card"],
            roles=[ROLES[_role]] if (_role := data.get("role")) else [],
            joined_at=datetime.fromtimestamp(data["join_time"]) if data["join_time"] else None,
            mute=(
                MuteInfo(muted=True, duration=timedelta(seconds=data["mute_duration"]))
                if "mute_duration" in data
                else None
            ),
        )

    async def query_user(self, bot: Bot, user_id: str):
        try:
            friend = await bot.get_friend(target=int(user_id))
            data = {
                "user_id": str(friend.id),
                "name": friend.nickname,
                "nickname": friend.remark,
            }
        except ActionFailed:
            usr = await bot.get_user_profile(target=int(user_id))
            data = {
                "user_id": user_id,
                "name": usr.nickname,
            }
        return self.extract_user(data)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            if user := await self.query_user(bot, scene_id):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "avatar": user.avatar,
                }
                return self.extract_scene(data)

        elif scene_type == SceneType.GROUP:
            group = await bot.get_group(target=int(scene_id))
            data = {
                "group_id": str(group.id),
                "group_name": group.name,
            }
            return self.extract_scene(data)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type == SceneType.GROUP:
            group_id = parent_scene_id
            member = await bot.get_member(group=int(group_id), target=int(user_id))
            data = {
                "group_id": group_id,
                "user_id": str(member.id),
                "name": member.name,
                "card": member.name,
                "role": member.permission,
                "join_time": member.join_timestamp,
                "mute_duration": member.mute_time,
            }
            try:
                info = await bot.get_member_profile(group=int(group_id), member=member.id)
                data["name"] = info.nickname
                data["gender"] = info.sex.lower()
            except ActionFailed:
                pass
            return self.extract_member(data, None)

    async def query_users(self, bot: Bot):
        friends = await bot.get_friend_list()
        for friend in friends:
            data = {
                "user_id": str(friend.id),
                "name": friend.nickname,
                "nickname": friend.remark,
            }
            yield self.extract_user(data)

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is not None and scene_type > SceneType.GROUP:
            return

        if scene_type is None or scene_type == SceneType.PRIVATE:
            async for user in self.query_users(bot):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "avatar": user.avatar,
                }
                yield self.extract_scene(data)
            if scene_type == SceneType.PRIVATE:
                return

        groups = await bot.get_group_list()
        for group in groups:
            data = {
                "group_id": str(group.id),
                "group_name": group.name,
            }
            yield self.extract_scene(data)

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        if scene_type != SceneType.GROUP:
            return

        group_id = parent_scene_id
        members = await bot.get_member_list(group=int(group_id))
        for member in members:
            try:
                info = await bot.get_member_profile(group=int(group_id), member=member.id)
                data = {
                    "group_id": group_id,
                    "user_id": str(member.id),
                    "name": info.nickname,
                    "card": member.name,
                    "role": member.permission,
                    "join_time": member.join_timestamp,
                    "gender": info.sex.lower(),
                    "mute_duration": member.mute_time,
                }
            except ActionFailed:
                data = {
                    "group_id": group_id,
                    "user_id": str(member.id),
                    "name": member.name,
                    "card": member.name,
                    "role": member.permission,
                    "join_time": member.join_timestamp,
                    "mute_duration": member.mute_time,
                }
            yield self.extract_member(data, None)

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.mirai,
            "scope": SupportScope.qq_client,
        }


fetcher = InfoFetcher(SupportAdapter.mirai)


@fetcher.supply
async def _(bot: Bot, event: FriendMessage):
    try:
        info = await bot.get_friend_profile(friend=event.sender)
        sex = info.sex.lower()
    except ActionFailed:
        sex = "unknown"
    return {
        "user_id": str(event.sender.id),
        "name": event.sender.nickname,
        "nickname": event.sender.remark,
        "gender": sex,
    }


@fetcher.supply
async def _(bot: Bot, event: StrangerMessage):
    try:
        info = await bot.get_user_profile(target=event.sender)
        sex = info.sex.lower()
    except ActionFailed:
        sex = "unknown"
    return {
        "user_id": str(event.sender.id),
        "name": event.sender.nickname,
        "nickname": event.sender.remark,
        "gender": sex,
    }


@fetcher.supply
async def _(bot: Bot, event: GroupMessage | TempMessage):
    try:
        member_info = await bot.get_member_profile(group=event.group.id, member=event.sender.id)
        nickname = member_info.nickname
    except ActionFailed:
        nickname = event.sender.name
    return {
        "group_id": str(event.group.id),
        "group_name": event.group.name,
        "user_id": str(event.sender.id),
        "name": nickname,
        "card": event.sender.name,
        "role": event.sender.permission,
        "join_time": event.sender.join_timestamp,
        "mute_duration": event.sender.mute_time,
    }


@fetcher.supply
async def _(bot: Bot, event: BotJoinGroupEvent):
    self_info = await bot.get_bot_profile()
    if not event.inviter:
        return {
            "user_id": str(bot.self_id),
            "name": self_info.nickname,
            "card": self_info.nickname,
            "role": event.group.account_perm,
        }
    try:
        inviter_info = await bot.get_member_profile(group=event.group.id, member=event.inviter.id)
        nickname = inviter_info.nickname
    except ActionFailed:
        nickname = event.inviter.name
    return {
        "group_id": str(event.group.id),
        "group_name": event.group.name,
        "user_id": str(bot.self_id),
        "name": self_info.nickname,
        "card": self_info.nickname,
        "role": event.group.account_perm,
        "operator": {
            "user_id": str(event.inviter.id),
            "name": nickname,
            "card": event.inviter.name,
            "role": event.inviter.permission,
            "join_time": event.inviter.join_timestamp,
            "mute_duration": event.inviter.mute_time,
        },
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: (
        BotMuteEvent
        | BotUnmuteEvent
        | BotLeaveEventKick
        | BotLeaveEventDisband
        | GroupNameChangeEvent
        | GroupEntranceAnnouncementChangeEvent
        | GroupMuteAllEvent
        | GroupAllowMemberInviteEvent
    ),
):
    self_info = await bot.get_bot_profile()
    if not event.operator:
        return {
            "user_id": str(bot.self_id),
            "name": self_info.nickname,
            "card": self_info.nickname,
            "role": event.group.account_perm,
        }
    try:
        operator_info = await bot.get_member_profile(group=event.group.id, member=event.operator.id)
        nickname = operator_info.nickname
    except ActionFailed:
        nickname = event.operator.name
    return {
        "group_id": str(event.group.id),
        "group_name": event.group.name,
        "user_id": str(bot.self_id),
        "name": self_info.nickname,
        "card": self_info.nickname,
        "role": event.group.account_perm,
        "mute_duration": getattr(event, "duration", 0),
        "operator": {
            "user_id": str(event.operator.id),
            "name": nickname,
            "card": event.operator.name,
            "role": event.operator.permission,
            "join_time": event.operator.join_timestamp,
            "mute_duration": event.operator.mute_time,
        },
    }


@fetcher.supply
async def _(
    bot: Bot, event: FriendAddEvent | FriendDeleteEvent | FriendInputStatusChangedEvent | FriendNickChangedEvent
):
    return {
        "user_id": str(event.friend.id),
        "name": event.friend.nickname,
        "nickname": event.friend.remark,
    }


@fetcher.supply
async def _(bot: Bot, event: FriendRecallEvent):
    try:
        info = await bot.get_friend(target=event.operator)
    except ActionFailed:
        info = event.friend
    return {
        "user_id": str(info.id),
        "name": info.nickname,
        "nickname": info.remark,
    }


@fetcher.supply
async def _(bot: Bot, event: GroupRecallEvent):
    try:
        member = await bot.get_member(group=event.group.id, target=event.author_id)
        member_info = await bot.get_member_profile(group=event.group.id, member=event.author_id)
        name = member_info.nickname
        card = member.name
        role = member.permission
    except ActionFailed:
        name = ""
        card = ""
        role = MemberPerm.Member
    if not event.operator:
        return {
            "group_id": str(event.group.id),
            "group_name": event.group.name,
            "user_id": str(event.author_id),
            "name": name,
            "card": card,
            "role": role,
        }
    try:
        operator_info = await bot.get_member_profile(group=event.group.id, member=event.operator)
        nickname = operator_info.nickname
    except ActionFailed:
        nickname = event.operator.name
    return {
        "group_id": str(event.group.id),
        "group_name": event.group.name,
        "user_id": str(event.author_id),
        "name": name,
        "card": card,
        "role": role,
        "operator": {
            "user_id": str(event.operator),
            "name": nickname,
            "card": event.operator,
            "role": member.permission,
            "join_time": member.join_timestamp,
            "mute_duration": member.mute_time,
        },
    }


@fetcher.supply
async def _(bot: Bot, event: NudgeEvent):
    scene_type = event.scene
    if scene_type == "friend":
        try:
            info = await bot.get_friend(target=event.supplicant)
            name = info.nickname
            nickname = info.remark
        except ActionFailed:
            name = ""
            nickname = ""
        return {
            "user_id": str(event.supplicant),
            "name": name,
            "nickname": nickname,
            "operator": {
                "user_id": str(event.supplicant),
                "name": name,
                "nickname": nickname,
            },
        }
    if scene_type == "stranger":
        try:
            info = await bot.get_user_profile(target=event.supplicant)
            name = info.nickname
        except ActionFailed:
            name = ""
        return {
            "user_id": str(event.supplicant),
            "name": name,
            "operator": {
                "user_id": str(event.supplicant),
                "name": name,
            },
        }
    if isinstance(event.subject, Group):
        try:
            member = await bot.get_member(group=event.subject.id, target=event.target)
            operator = await bot.get_member(group=event.subject.id, target=event.supplicant)
            member_info = await bot.get_member_profile(group=event.subject.id, member=event.target)
            operator_info = await bot.get_member_profile(group=event.subject.id, member=event.supplicant)
            name = member_info.nickname
            card = member.name
            role = member.permission
            operator_name = operator_info.nickname
            operator_card = operator.name
            operator_role = operator.permission
        except ActionFailed:
            name = ""
            card = ""
            role = MemberPerm.Member
            operator_name = ""
            operator_card = ""
            operator_role = MemberPerm.Member
        return {
            "group_id": str(event.subject.id),
            "group_name": event.subject.name,
            "user_id": str(event.target),
            "name": name,
            "card": card,
            "role": role,
            "operator": {
                "user_id": str(event.supplicant),
                "name": operator_name,
                "card": operator_card,
                "role": operator_role,
            },
        }
    raise NotImplementedError(f"Event {type(event)} not supported yet")


@fetcher.supply
async def _(bot: Bot, event: MemberJoinEvent):
    try:
        info = await bot.get_member_profile(group=event.group.id, member=event.member.id)
        name = info.nickname
    except ActionFailed:
        name = event.member.name
    if not event.inviter:
        return {
            "group_id": str(event.group.id),
            "group_name": event.group.name,
            "user_id": str(event.member.id),
            "name": name,
            "card": event.member.name,
            "role": event.member.permission,
            "join_time": event.member.join_timestamp,
            "mute_duration": event.member.mute_time,
        }
    try:
        inviter_info = await bot.get_member_profile(group=event.group.id, member=event.inviter.id)
        nickname = inviter_info.nickname
    except ActionFailed:
        nickname = event.inviter.name
    return {
        "group_id": str(event.group.id),
        "group_name": event.group.name,
        "user_id": str(event.member.id),
        "name": name,
        "card": event.member.name,
        "role": event.member.permission,
        "join_time": event.member.join_timestamp,
        "mute_duration": event.member.mute_time,
        "operator": {
            "user_id": str(event.inviter.id),
            "name": nickname,
            "card": event.inviter.name,
            "role": event.inviter.permission,
            "join_time": event.inviter.join_timestamp,
            "mute_duration": event.inviter.mute_time,
        },
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: (
        MemberLeaveEventKick
        | MemberLeaveEventQuit
        | MemberCardChangeEvent
        | MemberSpecialTitleChangeEvent
        | MemberPermissionChangeEvent
        | MemberMuteEvent
        | MemberUnmuteEvent
        | MemberHonorChangeEvent
    ),
):
    try:
        member_info = await bot.get_member_profile(group=event.group.id, member=event.member.id)
        name = member_info.nickname
    except ActionFailed:
        name = event.member.name
    if not (operator := getattr(event, "operator", None)):
        return {
            "group_id": str(event.group.id),
            "group_name": event.group.name,
            "user_id": str(event.member.id),
            "name": name,
            "card": event.member.name,
            "role": event.member.permission,
            "join_time": event.member.join_timestamp,
            "mute_duration": event.member.mute_time,
        }
    try:
        operator_info = await bot.get_member_profile(group=event.group.id, member=operator.id)
        nickname = operator_info.nickname
    except ActionFailed:
        nickname = operator.name
    return {
        "group_id": str(event.group.id),
        "group_name": event.group.name,
        "user_id": str(event.member.id),
        "name": name,
        "card": event.member.name,
        "role": event.member.permission,
        "join_time": event.member.join_timestamp,
        "mute_duration": event.member.mute_time,
        "operator": {
            "user_id": str(operator.id),
            "name": nickname,
            "card": operator.name,
            "role": operator.permission,
            "join_time": operator.join_timestamp,
            "mute_duration": operator.mute_time,
        },
    }


@fetcher.supply
async def _(bot: Bot, event: NewFriendRequestEvent):
    return {
        "user_id": str(event.supplicant),
        "name": event.nickname,
    }


@fetcher.supply
async def _(bot: Bot, event: MemberJoinRequestEvent | BotInvitedJoinGroupRequestEvent):
    return {
        "group_id": str(event.source_group),
        "group_name": event.group_name,
        "user_id": str(event.supplicant),
        "name": event.nickname,
    }



from datetime import timedelta

from nonebot.adapters.discord import Bot, is_not_unset, is_unset
from nonebot.adapters.discord.api.model import Channel as DiscordChannel
from nonebot.adapters.discord.api.model import GuildMember, Snowflake
from nonebot.adapters.discord.api.model import User as DiscordUser
from nonebot.adapters.discord.api.types import ChannelType as DiscordChannelType
from nonebot.adapters.discord.api.types import UNSET
from nonebot.adapters.discord.event import (
    ChannelCreateEvent,
    ChannelDeleteEvent,
    ChannelUpdateEvent,
    DirectMessageCreateEvent,
    DirectMessageDeleteBulkEvent,
    DirectMessageDeleteEvent,
    DirectMessageReactionAddEvent,
    DirectMessageReactionRemoveAllEvent,
    DirectMessageReactionRemoveEmojiEvent,
    DirectMessageReactionRemoveEvent,
    DirectMessageUpdateEvent,
    Event,
    GuildBanAddEvent,
    GuildBanRemoveEvent,
    GuildCreateEvent,
    GuildDeleteEvent,
    GuildMemberAddEvent,
    GuildMemberRemoveEvent,
    GuildMemberUpdateEvent,
    GuildMessageCreateEvent,
    GuildMessageDeleteBulkEvent,
    GuildMessageDeleteEvent,
    GuildMessageReactionAddEvent,
    GuildMessageReactionRemoveAllEvent,
    GuildMessageReactionRemoveEmojiEvent,
    GuildMessageReactionRemoveEvent,
    GuildMessageUpdateEvent,
    GuildUpdateEvent,
    InteractionCreateEvent,
)

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, MuteInfo, Role, Scene, SceneType, User

CHANNEL_TYPE = {
    DiscordChannelType.GUILD_TEXT: SceneType.CHANNEL_TEXT,
    DiscordChannelType.GUILD_VOICE: SceneType.CHANNEL_VOICE,
    DiscordChannelType.GUILD_CATEGORY: SceneType.CHANNEL_CATEGORY,
    DiscordChannelType.DM: SceneType.PRIVATE,
    DiscordChannelType.GROUP_DM: SceneType.GROUP,
    DiscordChannelType.GUILD_STAGE_VOICE: SceneType.CHANNEL_VOICE,
    DiscordChannelType.GUILD_DIRECTORY: SceneType.CHANNEL_CATEGORY,
}

BASE_URL = "https://cdn.discordapp.com/"


def avatar_url(id: str, avatar: str):
    if not avatar:
        return None
    return f"{BASE_URL}avatars/{id}/{avatar}.png?size=1024"


async def _handle_roles(bot: Bot, guild_id: str, roles: list[Snowflake]):
    if not roles:
        return [Role("MEMBER", 1, "member")]
    res = []
    resp = await bot.get_guild_roles(guild_id=int(guild_id))
    for role in resp:
        if role.id not in roles:
            continue
        perm = int(role.permissions)
        if perm & (1 << 3) == (1 << 3):
            if perm & (1 << 5) == (1 << 5):
                res.append(("OWNER", 100, role.name))
            res.append(("ADMINISTRATOR", 10, role.name))
        if perm & (1 << 4) == (1 << 4):
            res.append(("CHANNEL_ADMINISTRATOR", 9, role.name))
        res.append((str(role.id), 1, role.name))
    if not res:
        return [Role("MEMBER", 1, "member")]
    return [Role(*r) for r in res]


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            avatar=avatar_url(data["user_id"], data["avatar"]),
        )

    def extract_scene(self, data):
        if "guild_id" in data:
            if "channel_id" in data:
                return Scene(
                    id=data["channel_id"],
                    name=data.get("channel_name"),
                    type=data.get("channel_type", SceneType.CHANNEL_TEXT),
                    avatar=avatar_url(data["channel_id"], data.get("channel_avatar") or ""),
                    parent=Scene(
                        id=data["guild_id"],
                        name=data.get("guild_name"),
                        type=SceneType.GUILD,
                        avatar=avatar_url(data["guild_id"], data.get("guild_avatar") or ""),
                    ),
                )
            return Scene(
                id=data["guild_id"],
                name=data.get("guild_name"),
                type=SceneType.GUILD,
                avatar=avatar_url(data["guild_id"], data.get("guild_avatar") or ""),
            )
        if "channel_id" in data:
            return Scene(
                id=data["channel_id"],
                name=data.get("channel_name"),
                type=data.get("channel_type", SceneType.CHANNEL_TEXT),
                avatar=avatar_url(data["channel_id"], data.get("channel_avatar") or ""),
            )
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
            name=data["name"],
            avatar=avatar_url(data["user_id"], data.get("avatar") or ""),
        )

    def extract_member(self, data, user: User | None):
        if "guild_id" in data or "channel_id" in data:
            if user:
                return Member(user, nick=data["nickname"], roles=data.get("roles", []), joined_at=data.get("joined_at"))
            return Member(
                User(
                    id=data["user_id"],
                    name=data["name"],
                    avatar=avatar_url(data["user_id"], data.get("avatar") or ""),
                ),
                nick=data["nickname"],
                roles=data.get("roles", []),
                joined_at=data.get("joined_at"),
            )
        return None

    async def query_user(self, bot: Bot, user_id: str):
        if bot.self_id == user_id:
            user = await bot.get_current_user()
        else:
            user = await bot.get_user(user_id=int(user_id))
        data = {
            "user_id": str(user.id),
            "name": user.username,
            "avatar": user.avatar,
        }
        return self.extract_user(data)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            user = await bot.get_user(user_id=int(scene_id))
            return self.extract_scene(
                {
                    "user_id": user.id,
                    "name": user.username,
                    "avatar": user.avatar,
                }
            )

        elif scene_type == SceneType.GUILD:
            guild = await bot.get_guild(guild_id=int(scene_id))
            return self.extract_scene(
                {
                    "guild_id": str(guild.id),
                    "guild_name": guild.name,
                    "guild_avatar": guild.icon,
                }
            )

        elif scene_type >= SceneType.CHANNEL_TEXT:
            channel = await bot.get_channel(channel_id=int(scene_id))
            return self.extract_scene(
                {
                    "channel_id": str(channel.id),
                    "channel_name": channel.name,
                    "channel_type": CHANNEL_TYPE.get(channel.type, SceneType.CHANNEL_TEXT),
                    "channel_avatar": channel.icon,
                }
            )

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        guild_id = parent_scene_id

        member = await bot.get_guild_member(guild_id=int(guild_id), user_id=int(user_id))
        if isinstance(member.user, DiscordUser):
            user = User(
                id=str(member.user.id),
                name=member.user.username,
                avatar=member.user.avatar,
            )
            return Member(
                user=user,
                nick="" if member.nick is UNSET else member.nick,
                roles=await _handle_roles(bot, guild_id, member.roles),
                joined_at=member.joined_at,
                mute=None if member.mute is UNSET else MuteInfo(muted=member.mute, duration=timedelta(60)),
            )

    def query_users(self, bot: Bot):
        raise NotImplementedError

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type in (SceneType.PRIVATE, SceneType.GROUP):
            return

        guilds = await bot.get_current_user_guilds(limit=100)
        while guilds:
            for guild in guilds:
                if parent_scene_id is None or str(guild.id) == parent_scene_id:
                    _guild = Scene(
                        id=str(guild.id),
                        type=SceneType.GUILD,
                        name=guild.name,
                        avatar=avatar_url(str(guild.id), guild.icon or ""),
                    )
                    if scene_type is None or scene_type == SceneType.GUILD:
                        yield _guild
                    if scene_type == SceneType.GUILD:
                        continue
                    channels = await bot.get_guild_channels(guild_id=guild.id)
                    for channel in channels:
                        yield Scene(
                            id=str(channel.id),
                            type=CHANNEL_TYPE.get(channel.type, SceneType.CHANNEL_TEXT),
                            name=(channel.name if is_not_unset(channel.name) else None),
                            avatar=(
                                avatar_url(str(channel.id), channel.icon)
                                if is_not_unset(channel.icon) and channel.icon is not None
                                else None
                            ),
                            parent=_guild,
                        )
            if len(guilds) < 100:
                break
            guilds = await bot.get_current_user_guilds(limit=100, after=guilds[-1].id)

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        guild_id = parent_scene_id

        members = await bot.list_guild_members(guild_id=int(guild_id), limit=100)
        while members:
            for member in members:
                if isinstance(member.user, DiscordUser):
                    user = User(
                        id=str(member.user.id),
                        name=member.user.username,
                        avatar=member.user.avatar,
                    )
                else:
                    continue
                yield Member(
                    user=user,
                    nick="" if member.nick is UNSET else member.nick,
                    roles=await _handle_roles(bot, guild_id, member.roles),
                    joined_at=member.joined_at,
                    mute=None if member.mute is UNSET else MuteInfo(muted=member.mute, duration=timedelta(60)),
                )
            if len(members) < 100:
                break
            if is_unset(members[-1].user):
                raise ValueError("Discord member payload is missing user")
            members = await bot.list_guild_members(guild_id=int(guild_id), limit=100, after=members[-1].user.id)

    def supply_self(self, bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.discord,
            "scope": SupportScope.discord,
        }


fetcher = InfoFetcher(SupportAdapter.discord)


@fetcher.supply
async def _(bot: Bot, event: InteractionCreateEvent):
    if isinstance(event.user, DiscordUser):
        base = {
            "user_id": str(event.user.id),
            "name": event.user.username,
            "nickname": "",
            "avatar": event.user.avatar,
        }
        return base
    if is_unset(event.member):
        raise ValueError("Discord interaction payload is missing member")
    if is_unset(event.guild_id):
        raise ValueError("Discord interaction payload is missing guild_id")
    if is_unset(event.channel_id):
        raise ValueError("Discord interaction payload is missing channel_id")
    if is_unset(event.member.user):
        raise ValueError("Discord interaction member payload is missing user")
    guild_info = await bot.get_guild(guild_id=event.guild_id)
    base = {
        "user_id": str(event.member.user.id),
        "name": event.member.user.username,
        "nickname": event.member.nick,
        "avatar": event.member.user.avatar,
        "guild_id": str(event.guild_id),
        "guild_name": guild_info.name,
        "guild_avatar": guild_info.icon,
    }
    if isinstance(event.channel, DiscordChannel):
        base |= {
            "channel_id": str(event.channel.id),
            "channel_name": event.channel.name or "",
            "channel_type": CHANNEL_TYPE.get(event.channel.type, SceneType.CHANNEL_TEXT),
            "channel_avatar": event.channel.icon or "",
        }
    else:
        channel = await bot.get_channel(channel_id=event.channel_id)
        base |= {
            "channel_id": str(channel.id),
            "channel_name": channel.name or "",
            "channel_type": CHANNEL_TYPE.get(channel.type, SceneType.CHANNEL_TEXT),
            "channel_avatar": channel.icon or "",
        }
    return base


@fetcher.supply
async def _(
    bot: Bot,
    event: DirectMessageCreateEvent | GuildMessageCreateEvent | DirectMessageUpdateEvent | GuildMessageUpdateEvent,
):
    if is_unset(event.author):
        raise ValueError("Discord message payload is missing author")
    base = {
        "user_id": str(event.author.id),
        "name": event.author.username,
        "nickname": event.author.username,
        "avatar": event.author.avatar,
    }
    if isinstance(event.guild_id, Snowflake):
        guild = await bot.get_guild(guild_id=int(event.guild_id))
        base |= {
            "guild_id": str(event.guild_id),
            "guild_name": guild.name,
            "guild_avatar": guild.icon,
        }
        channel = await bot.get_channel(channel_id=int(event.channel_id))
        base |= {
            "channel_id": str(event.channel_id),
            "channel_name": channel.name,
            "channel_type": CHANNEL_TYPE.get(channel.type, SceneType.CHANNEL_TEXT),
            "channel_avatar": channel.icon,
        }
        if isinstance(event.member, GuildMember):
            base |= {
                "nickname": event.member.nick or event.author.username,
                "roles": await _handle_roles(bot, str(event.guild_id), event.member.roles),
                "joined_at": event.member.joined_at,
            }
        else:
            member = await bot.get_guild_member(guild_id=event.guild_id, user_id=event.author.id)
            base |= {
                "nickname": member.nick or event.author.username,
                "roles": await _handle_roles(bot, str(event.guild_id), member.roles),
                "joined_at": member.joined_at,
            }
    return base


@fetcher.supply
async def _(
    bot: Bot,
    event: (
        DirectMessageDeleteEvent
        | DirectMessageDeleteBulkEvent
        | GuildMessageDeleteEvent
        | GuildMessageDeleteBulkEvent
        | GuildMessageReactionRemoveAllEvent
        | DirectMessageReactionRemoveAllEvent
        | DirectMessageReactionRemoveEmojiEvent
        | GuildMessageReactionRemoveEmojiEvent
    ),
):
    self_info = await bot.get_current_user()
    base = {
        "user_id": str(self_info.id),
        "name": self_info.username,
        "nickname": self_info.username,
        "avatar": self_info.avatar,
    }
    if isinstance(event.guild_id, Snowflake):
        guild = await bot.get_guild(guild_id=int(event.guild_id))
        base |= {
            "guild_id": str(event.guild_id),
            "guild_name": guild.name,
            "guild_avatar": guild.icon,
        }
        channel = await bot.get_channel(channel_id=int(event.channel_id))
        base |= {
            "channel_id": str(event.channel_id),
            "channel_name": channel.name,
            "channel_type": CHANNEL_TYPE.get(channel.type, SceneType.CHANNEL_TEXT),
            "channel_avatar": channel.icon,
        }
    return base


@fetcher.supply
async def _(
    bot: Bot,
    event: (
        DirectMessageReactionAddEvent
        | DirectMessageReactionRemoveEvent
        | GuildMessageReactionAddEvent
        | GuildMessageReactionRemoveEvent
    ),
):
    user = await bot.get_user(user_id=event.user_id)
    base = {
        "user_id": str(event.user_id),
        "name": user.username,
        "nickname": user.username,
        "avatar": user.avatar,
    }
    if isinstance(event.guild_id, Snowflake):
        guild = await bot.get_guild(guild_id=int(event.guild_id))
        base |= {
            "guild_id": str(event.guild_id),
            "guild_name": guild.name,
            "guild_avatar": guild.icon,
        }
        channel = await bot.get_channel(channel_id=int(event.channel_id))
        base |= {
            "channel_id": str(event.channel_id),
            "channel_name": channel.name,
            "channel_type": CHANNEL_TYPE.get(channel.type, SceneType.CHANNEL_TEXT),
            "channel_avatar": channel.icon,
        }
        member = await bot.get_guild_member(guild_id=event.guild_id, user_id=event.user_id)
        base |= {
            "nickname": member.nick or user.username,
            "roles": await _handle_roles(bot, str(event.guild_id), member.roles),
            "joined_at": member.joined_at,
        }
    return base


@fetcher.supply_wildcard
async def _(bot: Bot, event: Event):
    if isinstance(event, (ChannelCreateEvent, ChannelDeleteEvent, ChannelUpdateEvent)):
        self_info = await bot.get_current_user()
        base = {
            "user_id": str(self_info.id),
            "name": self_info.username,
            "nickname": self_info.username,
            "avatar": self_info.avatar,
        }
        base |= {
            "channel_id": str(event.id),
            "channel_name": event.name,
            "channel_type": CHANNEL_TYPE.get(event.type, SceneType.CHANNEL_TEXT),
            "channel_avatar": event.icon,
        }
        if isinstance(event.guild_id, Snowflake):
            guild = await bot.get_guild(guild_id=int(event.guild_id))
            base |= {
                "guild_id": str(event.guild_id),
                "guild_name": guild.name,
                "guild_avatar": guild.icon,
            }
        return base
    if isinstance(event, (GuildBanAddEvent, GuildBanRemoveEvent)):
        base = {
            "user_id": str(event.user.id),
            "name": event.user.username,
            "nickname": event.user.username,
            "avatar": event.user.avatar,
        }
        guild = await bot.get_guild(guild_id=int(event.guild_id))
        base |= {
            "guild_id": str(event.guild_id),
            "guild_name": guild.name,
            "guild_avatar": guild.icon,
        }
        return base
    if isinstance(event, (GuildCreateEvent, GuildUpdateEvent)):
        self_info = await bot.get_current_user()
        base = {
            "user_id": str(self_info.id),
            "name": self_info.username,
            "nickname": self_info.username,
            "avatar": self_info.avatar,
        }
        base |= {
            "guild_id": str(event.id),
            "guild_name": event.name,
            "guild_avatar": event.icon,
        }
        return base
    if isinstance(event, GuildDeleteEvent):
        self_info = await bot.get_current_user()
        base = {
            "user_id": str(self_info.id),
            "name": self_info.username,
            "nickname": self_info.username,
            "avatar": self_info.avatar,
            "guild_id": str(event.id),
            "guild_name": "",
            "guild_avatar": "",
        }
        return base
    if isinstance(event, GuildMemberAddEvent):
        if is_unset(event.user):
            raise ValueError("Discord guild member add payload is missing user")
        base = {
            "user_id": str(event.user.id),
            "name": event.user.username,
            "nickname": event.nick or "",
            "avatar": event.avatar or event.user.avatar,
            "roles": await _handle_roles(bot, str(event.guild_id), event.roles),
            "joined_at": event.joined_at,
        }
        guild = await bot.get_guild(guild_id=int(event.guild_id))
        base |= {
            "guild_id": str(event.guild_id),
            "guild_name": guild.name,
            "guild_avatar": guild.icon,
        }
        return base
    if isinstance(event, GuildMemberUpdateEvent):
        base = {
            "user_id": str(event.user.id),
            "name": event.user.username,
            "nickname": event.nick or "",
            "avatar": event.user.avatar,
            "roles": await _handle_roles(bot, str(event.guild_id), event.roles),
            "joined_at": event.joined_at,
        }
        guild = await bot.get_guild(guild_id=int(event.guild_id))
        base |= {
            "guild_id": str(event.guild_id),
            "guild_name": guild.name,
            "guild_avatar": guild.icon,
        }
        return base
    if isinstance(event, GuildMemberRemoveEvent):
        base = {
            "user_id": str(event.user.id),
            "name": event.user.username,
            "nickname": event.user.username,
            "avatar": event.user.avatar,
        }
        guild = await bot.get_guild(guild_id=int(event.guild_id))
        base |= {
            "guild_id": str(event.guild_id),
            "guild_name": guild.name,
            "guild_avatar": guild.icon,
        }
        return base
    raise NotImplementedError



from datetime import datetime

from nonebot.adapters.milky import Bot
from nonebot.adapters.milky.event import (
    FriendMessageEvent,
    FriendNudgeEvent,
    FriendRequestEvent,
    GroupInvitationEvent,
    GroupInvitedJoinRequestEvent,
    GroupJoinRequestEvent,
    GroupMemberDecreaseEvent,
    GroupMemberIncreaseEvent,
    GroupMessageEvent,
    GroupMuteEvent,
    GroupNudgeEvent,
    MessageEvent,
    MessageRecallEvent,
    TempMessageEvent,
)
from nonebot.adapters.milky.event import Event as MilkyEvent
from nonebot.exception import ActionFailed
from nonebot.internal.adapter import Event

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Role, Scene, SceneType, User

ROLES = {
    "owner": ("OWNER", 100),
    "admin": ("ADMINISTRATOR", 10),
    "member": ("MEMBER", 1),
}


class InfoFetcher(BaseInfoFetcher):
    def get_session_id(self, event: Event) -> str:
        if isinstance(event, GroupNudgeEvent):
            return f"{event.get_session_id()}_{event.data.receiver_id}"
        return event.get_session_id()

    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data.get("name"),
            nick=data.get("nickname"),
            avatar=f"http://q1.qlogo.cn/g?b=qq&nk={data['user_id']}&s=640",
            gender=data.get("gender", "unknown"),
        )

    def extract_scene(self, data):
        if "group_id" not in data:
            return Scene(
                id=data["user_id"],
                type=SceneType.PRIVATE,
                name=data.get("name"),
                avatar=f"http://q1.qlogo.cn/g?b=qq&nk={data['user_id']}&s=640",
            )
        return Scene(
            id=data["group_id"],
            type=SceneType.GROUP,
            name=data.get("group_name"),
            avatar=f"https://p.qlogo.cn/gh/{data['group_id']}/{data['group_id']}/",
        )

    def extract_member(self, data, user: User | None):
        if "group_id" not in data:
            return None
        if user:
            return Member(
                user=user,
                nick=data.get("card"),
                roles=(
                    [Role(*ROLES[_role], name=_role)]
                    if (_role := data.get("role"))
                    else [Role(*ROLES["member"], name="member")]
                ),
                joined_at=datetime.fromtimestamp(data["join_time"]) if data.get("join_time") else None,
            )
        return Member(
            User(
                id=data["user_id"],
                name=data.get("name"),
                nick=data.get("nickname"),
                avatar=f"https://q2.qlogo.cn/headimg_dl?dst_uin={data['user_id']}&spec=640",
            ),
            nick=data.get("card"),
            roles=(
                [Role(*ROLES[_role], name=_role)]
                if (_role := data.get("role"))
                else [Role(*ROLES["member"], name="member")]
            ),
            joined_at=datetime.fromtimestamp(data["join_time"]) if data.get("join_time") else None,
        )

    async def query_user(self, bot: Bot, user_id: str) -> User:
        if user_id == bot.self_id:
            info = await bot.get_login_info()
            data = {"user_id": str(info.uin), "name": info.nickname, "nickname": info.nickname}
        else:
            info = await bot.get_user_profile(user_id=int(user_id))
            data = {
                "user_id": user_id,
                "name": info.nickname,
                "nickname": info.remark,
                "gender": info.sex,
            }
        return self.extract_user(data)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            if user := (await self.query_user(bot, scene_id)):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "avatar": user.avatar,
                }
                return self.extract_scene(data)

        elif scene_type == SceneType.GROUP:
            group = await bot.get_group_info(group_id=int(scene_id))
            data = {"group_id": group.group_id, "group_name": group.group_name}
            return self.extract_scene(data)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type != SceneType.GROUP:
            return
        group_id = parent_scene_id

        member = await bot.get_group_member_info(group_id=int(group_id), user_id=int(user_id))
        data = {
            "group_id": group_id,
            "user_id": str(member.user_id),
            "name": member.nickname,
            "card": member.card,
            "role": member.role,
            "join_time": member.join_time,
            "gender": member.sex,
        }
        return self.extract_member(data, None)

    async def query_users(self, bot: Bot):
        friends = await bot.get_friend_list()
        for friend in friends:
            data = {
                "user_id": str(friend.user_id),
                "name": friend.nickname,
                "nickname": friend.remark,
                "gender": friend.sex,
            }
            yield self.extract_user(data)

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is None or scene_type == SceneType.PRIVATE:
            async for user in self.query_users(bot):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "avatar": user.avatar,
                }
                yield self.extract_scene(data)

        if scene_type is None or scene_type == SceneType.GROUP:
            groups = await bot.get_group_list()
            for group in groups:
                data = {
                    "group_id": str(group.group_id),
                    "group_name": group.group_name,
                }
                yield self.extract_scene(data)

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        if scene_type != SceneType.GROUP:
            return
        group_id = parent_scene_id

        members = await bot.get_group_member_list(group_id=int(group_id))
        for member in members:
            data = {
                "group_id": group_id,
                "user_id": str(member.user_id),
                "name": member.nickname,
                "card": member.card,
                "role": member.role,
                "join_time": member.join_time,
                "gender": member.sex,
            }
            yield self.extract_member(data, None)

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.milky,
            "scope": SupportScope.qq_client,
        }


fetcher = InfoFetcher(SupportAdapter.milky)


@fetcher.supply
async def _(bot: Bot, event: MessageEvent | GroupMessageEvent | FriendMessageEvent | TempMessageEvent):
    if event.data.message_scene == "friend":
        assert event.data.friend
        return {
            "user_id": str(event.data.sender_id),
            "name": event.data.friend.nickname,
            "nickname": event.data.friend.remark,
            "gender": event.data.friend.sex,
        }
    assert event.data.group
    if event.data.message_scene == "temp":
        try:
            info = await bot.get_group_member_info(group_id=event.data.group.group_id, user_id=event.data.sender_id)
            base = {
                "user_id": str(event.data.sender_id),
                "name": info.nickname,
                "nickname": info.card,
                "gender": info.sex,
                "role": info.role,
                "join_time": info.join_time,
            }
        except ActionFailed:
            base = {"user_id": str(event.data.sender_id)}
    else:
        assert event.data.group_member
        base = {
            "user_id": str(event.data.sender_id),
            "name": event.data.group_member.nickname,
            "nickname": event.data.group_member.card,
            "gender": event.data.group_member.sex,
            "role": event.data.group_member.role,
            "join_time": event.data.group_member.join_time,
        }
    base |= {
        "group_id": str(event.data.group.group_id),
        "group_name": event.data.group.group_name,
    }
    return base


@fetcher.supply
async def _(bot: Bot, event: MessageRecallEvent):
    if event.data.message_scene == "friend":
        try:
            info = await bot.get_friend_info(user_id=event.data.sender_id)
            return {
                "user_id": str(event.data.sender_id),
                "name": info.nickname,
                "nickname": info.remark,
                "gender": info.sex,
            }
        except ActionFailed:
            return {"user_id": str(event.data.sender_id)}
    try:
        info = await bot.get_group_member_info(group_id=event.data.peer_id, user_id=event.data.sender_id)
        base = {
            "user_id": str(event.data.sender_id),
            "name": info.nickname,
            "nickname": info.card,
            "gender": info.sex,
            "role": info.role,
            "join_time": info.join_time,
        }
    except ActionFailed:
        base = {"user_id": str(event.data.sender_id)}
    try:
        info = await bot.get_group_info(group_id=event.data.peer_id)
        base |= {
            "group_id": str(event.data.peer_id),
            "group_name": info.group_name,
        }
    except ActionFailed:
        base["group_id"] = str(event.data.peer_id)
    return base


@fetcher.supply
async def _(bot: Bot, event: FriendNudgeEvent):
    try:
        info = await bot.get_friend_info(user_id=event.data.user_id)
        return {
            "user_id": str(event.data.user_id),
            "name": info.nickname,
            "nickname": info.remark,
            "gender": info.sex,
        }
    except ActionFailed:
        return {"user_id": str(event.data.user_id)}


@fetcher.supply
async def _(bot: Bot, event: GroupNudgeEvent):
    try:
        user = await bot.get_group_member_info(group_id=event.data.group_id, user_id=event.data.receiver_id)
        base: dict = {
            "user_id": str(event.data.receiver_id),
            "name": user.nickname,
            "nickname": user.card,
            "gender": user.sex,
            "role": user.role,
            "join_time": user.join_time,
        }
    except ActionFailed:
        base = {"user_id": str(event.data.receiver_id)}
    try:
        group = await bot.get_group_info(group_id=event.data.group_id)
        base |= {
            "group_id": str(event.data.group_id),
            "group_name": group.group_name,
        }
    except ActionFailed:
        base["group_id"] = str(event.data.group_id)
    try:
        operator = await bot.get_group_member_info(group_id=event.data.group_id, user_id=event.data.sender_id)
        base["operator"] = {
            "user_id": str(event.data.sender_id),
            "name": operator.nickname,
            "nickname": operator.card,
            "gender": operator.sex,
            "role": operator.role,
            "join_time": operator.join_time,
        }
    except ActionFailed:
        base["operator"] = {"user_id": str(event.data.sender_id)}
    return base


@fetcher.supply
async def _(bot: Bot, event: FriendRequestEvent):
    try:
        info = await bot.get_user_profile(user_id=event.data.initiator_id)
        return {
            "user_id": str(event.data.initiator_id),
            "name": info.nickname,
            "nickname": info.remark,
            "gender": info.sex,
        }
    except ActionFailed:
        return {"user_id": str(event.data.initiator_id)}


@fetcher.supply
async def _(bot: Bot, event: GroupJoinRequestEvent):
    try:
        user = await bot.get_user_profile(user_id=event.data.initiator_id)
        base: dict = {
            "user_id": str(event.data.initiator_id),
            "name": user.nickname,
            "nickname": user.remark,
            "gender": user.sex,
        }
    except ActionFailed:
        base = {"user_id": str(event.data.initiator_id)}
    try:
        group = await bot.get_group_info(group_id=event.data.group_id)
        base |= {
            "group_id": str(event.data.group_id),
            "group_name": group.group_name,
        }
    except ActionFailed:
        base["group_id"] = str(event.data.group_id)
    return base


@fetcher.supply
async def _(bot: Bot, event: GroupInvitedJoinRequestEvent):
    try:
        user = await bot.get_user_profile(user_id=event.data.target_user_id)
        base: dict = {
            "user_id": str(event.data.target_user_id),
            "name": user.nickname,
            "nickname": user.remark,
            "gender": user.sex,
        }
    except ActionFailed:
        base = {"user_id": str(event.data.target_user_id)}
    try:
        group = await bot.get_group_info(group_id=event.data.group_id)
        base |= {
            "group_id": str(event.data.group_id),
            "group_name": group.group_name,
        }
    except ActionFailed:
        base["group_id"] = str(event.data.group_id)
    if event.data:
        try:
            operator = await bot.get_group_member_info(group_id=event.data.group_id, user_id=event.data.initiator_id)
            base["operator"] = {
                "user_id": str(event.data.initiator_id),
                "name": operator.nickname,
                "nickname": operator.card,
                "gender": operator.sex,
            }
        except ActionFailed:
            base["operator"] = {"user_id": str(event.data.initiator_id)}
    return base


@fetcher.supply
async def _(bot: Bot, event: GroupInvitationEvent):
    try:
        user = await bot.get_user_profile(user_id=event.data.initiator_id)
        base = {
            "user_id": str(event.data.initiator_id),
            "name": user.nickname,
            "nickname": user.remark,
            "gender": user.sex,
        }
    except ActionFailed:
        base = {"user_id": str(event.data.initiator_id)}
    try:
        group = await bot.get_group_info(group_id=event.data.group_id)
        base |= {
            "group_id": str(event.data.group_id),
            "group_name": group.group_name,
        }
    except ActionFailed:
        base["group_id"] = str(event.data.group_id)
    return base


@fetcher.supply
async def _(bot: Bot, event: GroupMemberIncreaseEvent | GroupMemberDecreaseEvent | GroupMuteEvent):
    try:
        user = await bot.get_group_member_info(group_id=event.data.group_id, user_id=event.data.user_id)
        base: dict = {
            "user_id": str(event.data.user_id),
            "name": user.nickname,
            "nickname": user.card,
            "gender": user.sex,
            "role": user.role,
            "join_time": user.join_time,
        }
    except ActionFailed:
        try:
            user = await bot.get_user_profile(user_id=event.data.user_id)
            base = {
                "user_id": str(event.data.user_id),
                "name": user.nickname,
                "nickname": user.remark,
                "gender": user.sex,
                "role": "member",
            }
        except ActionFailed:
            base = {"user_id": str(event.data.user_id)}
    try:
        group = await bot.get_group_info(group_id=event.data.group_id)
        base |= {
            "group_id": str(event.data.group_id),
            "group_name": group.group_name,
        }
    except ActionFailed:
        base["group_id"] = str(event.data.group_id)
    if event.data.operator_id:
        try:
            operator = await bot.get_group_member_info(group_id=event.data.group_id, user_id=event.data.operator_id)
            base["operator"] = {
                "user_id": str(event.data.operator_id),
                "name": operator.nickname,
                "nickname": operator.card,
                "gender": operator.sex,
                "role": operator.role,
                "join_time": operator.join_time,
            }
        except ActionFailed:
            base["operator"] = {"user_id": str(event.data.operator_id)}
    return base


@fetcher.supply_wildcard
async def _(bot: Bot, evnet: MilkyEvent):
    try:
        user_id = evnet.get_user_id()
    except ValueError:
        user_id = None
    if evnet.is_private:
        assert user_id
        friend = await bot.get_friend_info(user_id=int(user_id))
        return {
            "user_id": str(user_id),
            "name": friend.nickname,
            "nickname": friend.remark,
            "gender": friend.sex,
        }
    group_id = getattr(evnet.data, "group_id", None)
    if not group_id:
        raise NotImplementedError(evnet)
    if not user_id:
        info = await bot.get_login_info()
        base = {
            "user_id": str(info.uin),
            "name": info.nickname,
        }
    else:
        info = await bot.get_group_member_info(group_id=int(group_id), user_id=int(user_id))
        base = {
            "user_id": user_id,
            "name": info.nickname,
            "nickname": info.card,
            "gender": info.sex,
            "role": info.role,
            "join_time": info.join_time,
        }
    group = await bot.get_group_info(group_id=int(group_id))
    base |= {
        "group_id": str(group.group_id),
        "group_name": group.group_name,
    }
    return base



from typing import TYPE_CHECKING, Literal, overload

from .constraint import SupportScope
from .model import BasicInfo, Member, Scene, Session, User

if TYPE_CHECKING:
    from nonebot_plugin_alconna import Target


@overload
def to_target(model: Session, *, without_self: bool = False) -> "Target": ...


@overload
def to_target(model: User | Member | Scene, info: BasicInfo) -> "Target": ...


@overload
def to_target(model: User | Member | Scene, info: str | SupportScope, *, without_self: Literal[True]) -> "Target": ...


def to_target(
    model: Session | User | Member | Scene,
    info: BasicInfo | str | SupportScope | None = None,
    without_self: bool = False,
) -> "Target":
    try:
        from nonebot_plugin_alconna import SupportAdapter as AlconnaSupportAdapter
        from nonebot_plugin_alconna import SupportScope as AlconnaSupportScope
        from nonebot_plugin_alconna import Target
    except ImportError:
        raise RuntimeError("Please install nonebot-plugin-alconna to use this function")
    if isinstance(model, Session):
        scene_id = model.scene.id
        scene_parent_id = model.scene.parent.id if model.scene.parent else ""
        if without_self:
            return Target(
                scene_id,
                scene_parent_id,
                model.scene.is_channel,
                model.scene.is_private,
                scope=AlconnaSupportScope(model.scope),
            )
        return Target(
            scene_id,
            scene_parent_id,
            model.scene.is_channel,
            model.scene.is_private,
            self_id=model.self_id,
            adapter=AlconnaSupportAdapter(model.adapter),
            scope=AlconnaSupportScope(model.scope),
        )
    if not info:
        raise ValueError("info is required when model is not Session")
    if not without_self and not isinstance(info, dict):
        raise ValueError("info must be BasicInfo when model is not Session and without_self is False")
    if isinstance(info, str):
        basic = {"scope": AlconnaSupportScope(info)}
    else:
        basic = {
            "self_id": info["self_id"],
            "adapter": AlconnaSupportAdapter(info["adapter"]),
            "scope": AlconnaSupportScope(info["scope"]),
        }
    if isinstance(model, User):
        return Target(
            model.id,
            private=True,
            **basic,  # type: ignore
        )
    elif isinstance(model, Member):
        return Target(
            model.user.id,
            private=True,
            **basic,  # type: ignore
        )
    elif isinstance(model, Scene):
        scene_id = model.id
        scene_parent_id = model.parent.id if model.parent else ""
        return Target(
            scene_id,
            scene_parent_id,
            model.is_channel,
            model.is_private,
            **basic,  # type: ignore
        )
    else:
        raise ValueError("model must be Session, User, Member or Scene")



from abc import ABCMeta, abstractmethod

from .constraint import SupportAdapter
from .fetch import InfoFetcher


class BaseLoader(metaclass=ABCMeta):
    @abstractmethod
    def get_adapter(self) -> SupportAdapter: ...

    @abstractmethod
    def get_fetcher(self) -> InfoFetcher: ...
